dnl
dnl  libflame
dnl  An object-based infrastructure for developing high-performance
dnl  dense linear algebra libraries.
dnl
dnl  Copyright (C) 2008, The University of Texas
dnl
dnl  libflame is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU Lesser General Public License as
dnl  published by the Free Software Foundation; either version 2.1 of
dnl  the License, or (at your option) any later version.
dnl
dnl  libflame is distributed in the hope that it will be useful, but
dnl  WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
dnl  Lesser General Public License for more details.
dnl
dnl  You should have received a copy of the GNU Lesser General Public
dnl  License along with libflame; if you did not receive a copy, see
dnl  http://www.gnu.org/licenses/.
dnl
dnl  For more information, please contact us at flame@cs.utexas.edu or
dnl  send mail to:
dnl
dnl  Field G. Van Zee and/or
dnl  Robert A. van de Geijn
dnl  The University of Texas at Austin
dnl  Department of Computer Sciences
dnl  1 University Station C0500
dnl  Austin TX 78712
dnl
AC_DEFUN([FLA_CHECK_ENABLE_PORTABLE_TIMER],
[
	dnl Tell the user we're checking whether to enable the option.
	AC_MSG_CHECKING([whether user requested a portable FLA_Clock() timer])
	
	dnl Determine whether the user gave the --enable-<option> or
	dnl --disable-<option>. If so, then run the first snippet of code;
	dnl otherwise, run the second code block.
	AC_ARG_ENABLE([portable-timer],
	              AC_HELP_STRING([--enable-portable-timer],[Define the FLA_Clock() timer function using getttimeofday() even if a more optimized inline assembly implementation is available for the build architecture. (By default, an architecture-specfic implementation is used if it exists; otherwise, the more portable gettimeofday()-based code is used.)]),
	[
		dnl If any form of the option is given, handle each case.
		if test "$enableval" = "no" ; then
			
			dnl User provided --enable-<option>=no or --disable-<option>.
			fla_enable_portable_timer=no

		elif test "$enableval" = "yes" ; then
			
			dnl User provided --enable-<option>=yes or --enable-<option>.
			fla_enable_portable_timer=yes
		else

			dnl We don't need an else branch because the configure script
			dnl should detect whether the user provided an unexpected argument
			dnl with the option.
			AC_MSG_ERROR([[Reached unreachable branch in FLA_CHECK_ENABLE_PORTABLE_TIMER!]])
		fi
	],
	[
			dnl User did not specify whether to enable or disable the portable
			dnl implementation of FLA_Clock(). Default behavior is to check for
			dnl non-portable architecture-specific implementation of FLA_Clock()
			dnl and use the (more portable) gettimeofday() implementation if an
			dnl architecture-specific version of the function cannot be found.
			fla_enable_portable_timer=no
	]
	)
	
	dnl Now act according to whether the option was requested.
	if   test "$fla_enable_portable_timer" = "yes" ; then
		
		dnl Output the result.
		AC_MSG_RESULT([yes])
		
		dnl Verify that we have gettimeofday() on hand.
		AC_CHECK_FUNC([gettimeofday])
		
		dnl Define the macro.
		AC_DEFINE(FLA_ENABLE_PORTABLE_DCLOCK,1,
		          [Determines whether to define a portable FLA_Clock() in terms of gettimeofday() from time.h.])
		
	elif test "$fla_enable_portable_timer" = "no" ; then
		
		dnl Output the result.
		AC_MSG_RESULT([no])
		
		dnl We'll need strchr(), which is used by detect_clocks() in
		dnl FLA_Clock.c. Verify that we have it.
		AC_CHECK_FUNC([strchr],[],
		[
			AC_MSG_ERROR([Failed to find a working version of strchr()! Try enabling the portable timer, which does not need strchr(), and rerun configure.])
		])
		
	else
		dnl Only "yes" and "no" are accepted, so this block is empty.
		AC_MSG_ERROR([[Reached unreachable branch in FLA_CHECK_ENABLE_PORTABLE_TIMER!]])
	fi
	
	dnl Substitute the output variable.
	AC_SUBST(fla_enable_portable_timer)

])
