dnl
dnl  libflame
dnl  An object-based infrastructure for developing high-performance
dnl  dense linear algebra libraries.
dnl
dnl  Copyright (C) 2008, The University of Texas
dnl
dnl  libflame is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU Lesser General Public License as
dnl  published by the Free Software Foundation; either version 2.1 of
dnl  the License, or (at your option) any later version.
dnl
dnl  libflame is distributed in the hope that it will be useful, but
dnl  WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
dnl  Lesser General Public License for more details.
dnl
dnl  You should have received a copy of the GNU Lesser General Public
dnl  License along with libflame; if you did not receive a copy, see
dnl  http://www.gnu.org/licenses/.
dnl
dnl  For more information, please contact us at flame@cs.utexas.edu or
dnl  send mail to:
dnl
dnl  Field G. Van Zee and/or
dnl  Robert A. van de Geijn
dnl  The University of Texas at Austin
dnl  Department of Computer Sciences
dnl  1 University Station C0500
dnl  Austin TX 78712
dnl
dnl 
dnl  fla_check_with_blas.m4
dnl  
dnl  This autoconf macro was based on the acx_blas macro written by
dnl  Steven G. Johnson <stevenj@alum.mit.edu>.
dnl 
AC_DEFUN([FLA_CHECK_WITH_BLAS],
[
	AC_REQUIRE([AC_F77_LIBRARY_LDFLAGS])
	
	dnl Begin with the library not yet found.
	fla_ref_blas_found=no
	
	dnl Allow the user to specify their favorite BLAS library. This only applies
	dnl to building the test suite executables, though.
	AC_ARG_WITH([blas], 
                 AC_HELP_STRING([--with-blas=lib],[Use BLAS library <lib> when linking the test suite. Note that <lib> is the base name of the library. To link with libxyz.so or libxyz.a, use xyz as the <lib> argument. If libxyz.so exists in a nonstandard directory <xyz_path> and/or needs an additional library <extra_lib> to link properly, you must specify these by setting the FLA_REF_BLAS_LDFLAGS and FLA_REF_BLAS_LIBS environment variables to "-L<xyz_path>" and "-l<extra_lib>" respectively before running configure.]),
	[
		dnl Handle the possible results assuming --with-blas was given.
		case $with_blas in
			yes | "")
				fla_ref_blas_arg=yes
				;;
			no)
				fla_ref_blas_arg=no
				;;
			-* | */* | *.a | *.so | *.so.* | *.o)
				fla_ref_blas_arg=absolute
				;;
			*)
				fla_ref_blas_arg=name
				;;
		esac
	
	],
	[
		dnl If --with-blas was not given, then assume we're looking for
		dnl the library as if the user has given --with-blas=blas.
		fla_ref_blas_arg=name
		with_blas='blas'
	])

	dnl Get fortran linker names of BLAS functions to check for.
	AC_F77_FUNC(dgemm)
	
	dnl Save the LDFLAGS AND LIBS variables. We might change them while testing
	dnl for the presence of BLAS libraries. This way, we'll be able to restore 
	dnl them to their original values later.
	fla_blas_save_LDFLAGS="$LDFLAGS"
	fla_blas_save_LIBS="$LIBS"
		
	dnl Add $FLIBS to the $LIBS environment variable temporarily while we invoke
	dnl checks for dgemm in various libraries. If $FLIBS contains any necessary
	dnl runtime/intrinsic libraries, then we'll need them to prevent unresolved
	dnl symbols during the test links.
	LIBS="$LIBS $FLIBS"

	dnl Handle each case parsed above.
	if   test "$fla_ref_blas_arg" = "no" ; then
		
		dnl Disallow the user from disabling the BLAS.
		AC_MSG_WARN([[You will not be able to run any test suites with BLAS disabled! You should rerun configure, but do not give --with-blas=no or --without-blas.]])
		
	elif test "$fla_ref_blas_arg" = "absolute" ; then
		
		dnl Only check for the existence of $with_blas if we are not cross compiling
		dnl The CHECK_FILE macro below results in a fatal error if it is invoked while
		dnl cross compiling.
		if test "$cross_compiling" = "yes" ; then
		
			dnl Check that the library whose absolute path was given exists.
			dnl and warn that its contenst will not be checked.
			AC_CHECK_FILE([$with_blas],
			[
				AC_MSG_WARN([[The BLAS library file $with_blas will NOT be checked for valid BLAS routines. We're going to continue under the assumption that it's a valid BLAS library. If you'd like configure to check the library contents of libxyz.so or libxyz.a, rerun configure with --with-blas=xyz. If libxyz exists in a nonstandard directory, then set the environment variables FLA_REF_BLAS_LDFLAGS and/or FLA_REF_BLAS_LIBS to "-L<xyz_path>" and "-l<extra_lib>" respectively before running configure.]])
				fla_ref_blas_found=yes
			],
			[
				AC_MSG_WARN([[The library $with_blas does not exist.]])
				AC_MSG_NOTICE([[Continuing search for suitable BLAS...]])
			])
		else
			dnl Assume the directory is valid if we're cross compiling
			AC_MSG_NOTICE([not allowed to check for existence of $with_blas while cross compiling.])
			AC_MSG_NOTICE([we're going to assume that $with_blas exists!])
			fla_ref_blas_found=yes
		fi
	else
		
		dnl Set the BLAS library to check for based on how we got here. If one was
		dnl specified, use it. Otherwise, assume it's named "blas".
		if test "$fla_ref_blas_arg" = "name" ; then
			blas_lib=$with_blas
		elif test "$fla_ref_blas_arg" = "yes" ; then
			blas_lib='blas'
			AC_MSG_NOTICE([A BLAS library was requested, but not specified by name; assuming 'blas'.])
		fi
		
		dnl If FLA_REF_BLAS_LDFLAGS is non-empty, then add it to LDFLAGS.
		if test "$FLA_REF_BLAS_LDFLAGS" != "" ; then
			AC_MSG_NOTICE([adding $FLA_REF_BLAS_LDFLAGS to LDFLAGS environment variable.])
			LDFLAGS="$FLA_REF_BLAS_LDFLAGS $LDFLAGS"
		fi
		
		dnl If FLA_REF_BLAS_LIBS is non-empty, then add it to LIBS.
		if test "$FLA_REF_BLAS_LIBS" != "" ; then
			AC_MSG_NOTICE([adding $FLA_REF_BLAS_LIBS to LIBS environment variable.])
			LIBS="$FLA_REF_BLAS_LIBS $LIBS"
		fi
		
		dnl Check whether we can link \$dgemm to the blas library specified.
		AC_CHECK_LIB($blas_lib, $dgemm,
		[
			fla_ref_blas_found=yes
			fla_ref_blas_lib_found="-l$blas_lib"
		],
		[
			AC_MSG_WARN([[Could not link against library -l$blas_lib.]])
			AC_MSG_NOTICE([[Some libraries (ie: libgoto) need extra linker arguments in LDFLAGS.]])
			AC_MSG_NOTICE([[Continuing search for suitable BLAS...]])
			fla_ref_blas_found=no
			fla_ref_blas_lib_found=''
		])dnl,
		dnl[$FLA_REF_BLAS_LIBS])
	fi

	dnl BLAS linked to by default?  (happens on some supercomputers)
	if test "$fla_ref_blas_found" = "no" ; then
		
		local_save_LIBS="$LIBS" ; LIBS="$LIBS"
		AC_CHECK_FUNC($dgemm, [fla_ref_blas_found=yes])
		LIBS="$local_save_LIBS"
	fi

	dnl BLAS in Kazushige Goto's libgoto?
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(goto, $dgemm,
		[
			fla_ref_blas_found=yes
			fla_ref_blas_lib_found="-lgoto -lpthread"
		],
		[],
		[-lpthread])
	fi
	
	dnl BLAS in Intel MKL?
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(mkl, $dgemm,
		[
			fla_ref_blas_found=yes
			fla_ref_blas_lib_found="-lmkl -lguide"
		],
		[],
		[-lguide])
	fi

	dnl BLAS in HP MLIB?
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(mlib, $dgemm,
		[
			fla_ref_blas_found=yes
			fla_ref_blas_lib_found="-lveclib"
		])
	fi

	dnl BLAS in IBM ESSL library? (requires generic BLAS lib, too)
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(essl, $dgemm,
		[
			AC_CHECK_LIB(blas, $dgemm,
			[
				fla_ref_blas_found=yes
				fla_ref_blas_lib_found="-lessl -lblas"
			],
			[],
			[-lessl])
		])
	fi

	dnl BLAS in Sun Performance library?
	if test "$fla_ref_blas_found" = "no" ; then
		
		dnl SPL only works with Sun CC, so let's disallow gcc
		if test "$GCC" != "yes" ; then
		
			AC_CHECK_LIB(sunmath, acosp,
			[
				AC_CHECK_LIB(sunperf, $dgemm,
				[
					fla_ref_blas_found=yes
					fla_ref_blas_lib_found="-xlic_lib=sunperf -lsunmath"
				],
				[],
				[-lsunmath])
			])
		fi
	fi

	dnl BLAS in ATLAS library? (http://math-atlas.sourceforge.net/)
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(atlas, ATL_xerbla,
		[
			AC_CHECK_LIB(f77blas, $dgemm,
			[
				AC_CHECK_LIB(cblas, cblas_dgemm,
		          [
					fla_ref_blas_found=yes
					fla_ref_blas_lib_found="-lcblas -lf77blas -latlas"
				],
				[],
				[-lf77blas -latlas])
			],
			[],
			[-latlas])
		])
	fi

	dnl BLAS in Alpha CXML library?
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(cxml, $dgemm,
		[
			fla_ref_blas_found=yes
			fla_ref_blas_lib_found="-lcxml"
		])
	fi

	dnl BLAS in Alpha DXML library? (now called CXML, see above)
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(dxml, $dgemm, 
		[
			fla_ref_blas_found=yes
			fla_ref_blas_lib_found="-ldxml"
		])
	fi

	dnl BLAS in SCSL library?  (SGI/Cray Scientific Library)
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(scs, $dgemm, 
		[
			fla_ref_blas_found=yes
			fla_ref_blas_lib_found="-lscs"
		])
	fi

	dnl BLAS in SGIMATH library?
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(complib.sgimath, $dgemm,
		[
			fla_ref_blas_found=yes
			fla_ref_blas_lib_found="-lcomplib.sgimath"
		])
	fi

	dnl Generic BLAS library?
	if test "$fla_ref_blas_found" = "no" ; then
		
		AC_CHECK_LIB(blas, $dgemm,
		[
			fla_ref_blas_found=yes
			fla_ref_blas_lib_found="-lblas"
		])
	fi

	dnl Set the fla_ref_blas_libs variable with the BLAS library found,
	dnl any additional libraries provided by the user in FLA_REF_BLAS_LIBS,
	dnl and $FLIBS, which are the runtime/intrinsic library flags that may
	dnl have been needed to get the BLAS library to link correctly.
	fla_ref_blas_libs="$fla_ref_blas_lib_found $FLA_REF_BLAS_LIBS $FLIBS"
	
	dnl Set the fla_ref_blas_ldflags variable to the value of FLA_REF_BLAS_LDFLAGS
	dnl which may have been set by the user to provide additional link paths.
	fla_ref_blas_ldflags="$FLA_REF_BLAS_LDFLAGS"
	
	dnl Substitute into the libs output variable.
	AC_SUBST(fla_ref_blas_libs)
	
	dnl Substitute into the ldflags output variable.
	AC_SUBST(fla_ref_blas_ldflags)
	
	dnl Restore the original value for LIBS and LDFLAGS.
	LDFLAGS="$fla_blas_save_LDFLAGS"
	LIBS="$fla_blas_save_LIBS"
	
	dnl Finally, execute ACTION-IF-FOUND/ACTION-IF-NOT-FOUND:
	if test "$fla_ref_blas_found" = "yes" ; then
		
		ifelse([$1],,AC_DEFINE(HAVE_BLAS,1,[Define if you have a BLAS library.]),[$1])
		:
	else
		fla_ref_blas_found=no
		AC_MSG_WARN([[Cannot properly configure test suite without BLAS! If you need to run the test suite, rerun configure, but provide a working blas with --with-blas=/path/to/libxyz.a or --with-blas=xyz where xyz is the name of the library. If the latter form is chosen, you must also set the environment variables FLA_REF_BLAS_LDFLAGS and/or FLA_REF_BLAS_LIBS to "-L<xyz_path>" and "-l<extra_lib>" respectively before running configure.]])
		$2
	fi
])
