dnl
dnl  libflame
dnl  An object-based infrastructure for developing high-performance
dnl  dense linear algebra libraries.
dnl
dnl  Copyright (C) 2008, The University of Texas
dnl
dnl  libflame is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU Lesser General Public License as
dnl  published by the Free Software Foundation; either version 2.1 of
dnl  the License, or (at your option) any later version.
dnl
dnl  libflame is distributed in the hope that it will be useful, but
dnl  WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
dnl  Lesser General Public License for more details.
dnl
dnl  You should have received a copy of the GNU Lesser General Public
dnl  License along with libflame; if you did not receive a copy, see
dnl  http://www.gnu.org/licenses/.
dnl
dnl  For more information, please contact us at flame@cs.utexas.edu or
dnl  send mail to:
dnl
dnl  Field G. Van Zee and/or
dnl  Robert A. van de Geijn
dnl  The University of Texas at Austin
dnl  Department of Computer Sciences
dnl  1 University Station C0500
dnl  Austin TX 78712
dnl
AC_DEFUN([FLA_SET_F77_OPENMP_FLAGS],
[
	AC_REQUIRE([FLA_OBSERVE_HOST_CPU_TYPE])
		
	dnl Echo Fortran OpenMP flags to user.
	AC_MSG_CHECKING([for (guessing) OpenMP flags for $F77])

	dnl Set the OpenMP compiler flags based on which Fortran compiler we're going to use.
	case $F77 in
		dnl Intel ifort.
		ifort)
			fla_f77_openmp_flags='-openmp'
		;;
		dnl PathScale pathcc
		pathf*)
			fla_f77_openmp_flags='-mp'
		;;
		dnl NEC sxf90.
		sxf90)
			fla_f77_openmp_flags='-P openmp'
		;;
		dnl IBM xlf.
		xlf*)
			fla_f77_openmp_flags='-qsmp=omp'
		;;
		dnl for all other Fortran compilers.
		*)
			fla_f77_openmp_flags='unknown'
		;;
	esac

	dnl Output the result.
	AC_MSG_RESULT([$fla_f77_openmp_flags])
			
	dnl Check string in case Fortran OpenMP compiler is unknown.
	if test "$fla_f77_openmp_flags" = "unknown" ; then
		
		dnl Tell the user we can't continue unless we know what flags
		dnl to pass to the Fortran compiler to enable OpenMP.
		AC_MSG_ERROR([configure doesn't know what flag to give $F77 in order to enable OpenMP support. Please submit a bug report to the FLAME developers at FLA_BUG_REPORT_ADDRESS.])
	fi

	dnl Output the Fortran OpenMP flags variable.
	AC_SUBST(fla_f77_openmp_flags)
])
