dnl
dnl  libflame
dnl  An object-based infrastructure for developing high-performance
dnl  dense linear algebra libraries.
dnl
dnl  Copyright (C) 2008, The University of Texas
dnl
dnl  libflame is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU Lesser General Public License as
dnl  published by the Free Software Foundation; either version 2.1 of
dnl  the License, or (at your option) any later version.
dnl
dnl  libflame is distributed in the hope that it will be useful, but
dnl  WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
dnl  Lesser General Public License for more details.
dnl
dnl  You should have received a copy of the GNU Lesser General Public
dnl  License along with libflame; if you did not receive a copy, see
dnl  http://www.gnu.org/licenses/.
dnl
dnl  For more information, please contact us at flame@cs.utexas.edu or
dnl  send mail to:
dnl
dnl  Field G. Van Zee and/or
dnl  Robert A. van de Geijn
dnl  The University of Texas at Austin
dnl  Department of Computer Sciences
dnl  1 University Station C0500
dnl  Austin TX 78712
dnl
AC_DEFUN([FLA_SET_F77_OPT_FLAGS],
[
	AC_REQUIRE([FLA_OBSERVE_HOST_CPU_TYPE])
	
	AC_MSG_CHECKING([for (guessing) appropriate $F77 optimization flags])

	if test "$1" == "yes" ; then

		dnl Set f77 compiler flags assuming we found...
		case $F77 in
			dnl GNU g77.
			g77)
				fla_f77_opt_flags='-O3'
			;;
			dnl GNU gfortran.
			gfortran)
				fla_f77_opt_flags='-O3'
			;;
			dnl Intel ifort.
			ifort)
				fla_f77_opt_flags='-O3'
			;;
			dnl PathScale pathf95.
			pathf95)
				fla_f77_opt_flags='-O3'
			;;
			dnl NEC sxf90.
			sxf90)
				fla_f77_opt_flags='-C vopt -Wf,-pvctl,noassume,vwork=stack'
			;;
			dnl IBM xlf.
			xlf*)
				fla_f77_opt_flags='-O5'
			;;
			dnl GNU gfortran for Cell PPU and SPU.
			ppu-gfortran)
				fla_f77_opt_flags='-O3'
			;;
			dnl ambiguous f77.
			f77)
				fla_f77_opt_flags='-O'
			;;
			dnl for all other Fortran compilers.
			*)
				fla_f77_opt_flags=''
			;;
		esac
	else
	
		dnl Set f77 compiler flags assuming we found...
		case $F77 in
			dnl GNU g77.
			g77)
				fla_f77_opt_flags='-O0'
			;;
			dnl GNU gfortran.
			gfortran)
				fla_f77_opt_flags='-O0'
			;;
			dnl Intel ifort.
			ifort)
				fla_f77_opt_flags='-O0'
			;;
			dnl PathScale pathf95.
			pathf95)
				fla_f77_opt_flags='-O0'
			;;
			dnl NEC sxf90.
			sxf90)
				fla_f77_opt_flags='-C noopt'
			;;
			dnl IBM xlf or xlf90.
			xlf*)
				fla_f77_opt_flags=''
			;;
			dnl GNU gfortran for Cell PPU and SPU.
			ppu-gfortran)
				fla_f77_opt_flags=''
			;;
			dnl ambiguous f77.
			f77)
				fla_f77_opt_flags=''
			;;
			dnl for all other Fortran compilers.
			*)
				fla_f77_opt_flags=''
			;;
		esac
	fi
	
	dnl Output the result.
	AC_MSG_RESULT([$fla_f77_opt_flags])
	
	dnl Substitute the optimization flags into the autoconf output files.
	AC_SUBST(fla_f77_opt_flags)

])
