/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

typedef struct FLA_Blocksize_s
{
	int s;
	int d;
	int c;
	int z;
} fla_blocksize_t;



struct FLA_Gemm_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Gemm_s* sub_gemm;
};
typedef struct FLA_Gemm_s fla_gemm_t;



struct FLA_Hemm_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Hemm_s* sub_hemm;
	struct FLA_Gemm_s* sub_gemm1;
	struct FLA_Gemm_s* sub_gemm2;
};
typedef struct FLA_Hemm_s fla_hemm_t;



struct FLA_Herk_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Herk_s* sub_herk;
	struct FLA_Gemm_s* sub_gemm;
};
typedef struct FLA_Herk_s fla_herk_t;



struct FLA_Her2k_s
{
	FLA_Matrix_type     matrix_type;
	int                 variant;
	fla_blocksize_t*    blocksize;
	struct FLA_Her2k_s* sub_her2k;
	struct FLA_Gemm_s*  sub_gemm1;
	struct FLA_Gemm_s*  sub_gemm2;
};
typedef struct FLA_Her2k_s fla_her2k_t;



struct FLA_Symm_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Symm_s* sub_symm;
	struct FLA_Gemm_s* sub_gemm1;
	struct FLA_Gemm_s* sub_gemm2;
};
typedef struct FLA_Symm_s fla_symm_t;



struct FLA_Syrk_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Syrk_s* sub_syrk;
	struct FLA_Gemm_s* sub_gemm;
};
typedef struct FLA_Syrk_s fla_syrk_t;



struct FLA_Syr2k_s
{
	FLA_Matrix_type     matrix_type;
	int                 variant;
	fla_blocksize_t*    blocksize;
	struct FLA_Syr2k_s* sub_syr2k;
	struct FLA_Gemm_s*  sub_gemm1;
	struct FLA_Gemm_s*  sub_gemm2;
};
typedef struct FLA_Syr2k_s fla_syr2k_t;



struct FLA_Trmm_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Trmm_s* sub_trmm;
	struct FLA_Gemm_s* sub_gemm;
};
typedef struct FLA_Trmm_s fla_trmm_t;



struct FLA_Trsm_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Trsm_s* sub_trsm;
	struct FLA_Gemm_s* sub_gemm;
};
typedef struct FLA_Trsm_s fla_trsm_t;



struct FLA_Chol_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Chol_s* sub_chol;
	struct FLA_Syrk_s* sub_syrk;
	struct FLA_Herk_s* sub_herk;
	struct FLA_Trsm_s* sub_trsm;
	struct FLA_Gemm_s* sub_gemm;
};
typedef struct FLA_Chol_s fla_chol_t;



struct FLA_Ttmm_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Ttmm_s* sub_ttmm;
	struct FLA_Syrk_s* sub_syrk;
	struct FLA_Herk_s* sub_herk;
	struct FLA_Trmm_s* sub_trmm;
	struct FLA_Gemm_s* sub_gemm;
};
typedef struct FLA_Ttmm_s fla_ttmm_t;



struct FLA_LU_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_LU_s*   sub_lu;
	struct FLA_Trsm_s* sub_trsm1;
	struct FLA_Trsm_s* sub_trsm2;
	struct FLA_Gemm_s* sub_gemm1;
	struct FLA_Gemm_s* sub_gemm2;
	struct FLA_Gemm_s* sub_gemm3;
};
typedef struct FLA_LU_s fla_lu_t;



struct FLA_QR_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_QR_s*   sub_qr;
	struct FLA_Trmm_s* sub_trmm1;
	struct FLA_Trmm_s* sub_trmm2;
	struct FLA_Trmm_s* sub_trmm3;
	struct FLA_Gemm_s* sub_gemm1;
	struct FLA_Gemm_s* sub_gemm2;
};
typedef struct FLA_QR_s fla_qr_t;



struct FLA_LQ_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_LQ_s*   sub_lq;
	struct FLA_Trmm_s* sub_trmm1;
	struct FLA_Trmm_s* sub_trmm2;
	struct FLA_Trmm_s* sub_trmm3;
	struct FLA_Gemm_s* sub_gemm1;
	struct FLA_Gemm_s* sub_gemm2;
};
typedef struct FLA_LQ_s fla_lq_t;



struct FLA_Hess_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Hess_s* sub_hess;
	struct FLA_Trmm_s* sub_trmm1;
	struct FLA_Trmm_s* sub_trmm2;
	struct FLA_Trmm_s* sub_trmm3;
	struct FLA_Trmm_s* sub_trmm4;
	struct FLA_Gemm_s* sub_gemm1;
	struct FLA_Gemm_s* sub_gemm2;
	struct FLA_Gemm_s* sub_gemm3;
};
typedef struct FLA_Hess_s fla_hess_t;



struct FLA_Trinv_s
{
	FLA_Matrix_type     matrix_type;
	int                 variant;
	fla_blocksize_t*    blocksize;
	struct FLA_Trinv_s* sub_trinv;
	struct FLA_Gemm_s*  sub_gemm;
	struct FLA_Trmm_s*  sub_trmm;
	struct FLA_Trsm_s*  sub_trsm1;
	struct FLA_Trsm_s*  sub_trsm2;
};
typedef struct FLA_Trinv_s fla_trinv_t;



struct FLA_Sylv_s
{
	FLA_Matrix_type     matrix_type;
	int                 variant;
	fla_blocksize_t*    blocksize;
	struct FLA_Sylv_s*  sub_sylv1;
	struct FLA_Sylv_s*  sub_sylv2;
	struct FLA_Sylv_s*  sub_sylv3;
	struct FLA_Gemm_s*  sub_gemm1;
	struct FLA_Gemm_s*  sub_gemm2;
	struct FLA_Gemm_s*  sub_gemm3;
	struct FLA_Gemm_s*  sub_gemm4;
	struct FLA_Gemm_s*  sub_gemm5;
	struct FLA_Gemm_s*  sub_gemm6;
	struct FLA_Gemm_s*  sub_gemm7;
	struct FLA_Gemm_s*  sub_gemm8;
};
typedef struct FLA_Sylv_s fla_sylv_t;



struct FLA_Accum_S_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
};
typedef struct FLA_Accum_S_s fla_accum_S_t;



struct FLA_Transpose_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Transpose_s* sub_trans;
	struct FLA_Swap_s* sub_swap;
};
typedef struct FLA_Transpose_s fla_transpose_t;



struct FLA_Swap_s
{
	FLA_Matrix_type    matrix_type;
	int                variant;
	fla_blocksize_t*   blocksize;
	struct FLA_Swap_s* sub_swap;
};
typedef struct FLA_Swap_s fla_swap_t;



struct FLA_SPDinv_s
{
	FLA_Matrix_type     matrix_type;
	int                 variant;
	fla_blocksize_t*    blocksize;
	struct FLA_Chol_s*  sub_chol;
	struct FLA_Trinv_s* sub_trinv;
	struct FLA_Ttmm_s*  sub_ttmm;
};
typedef struct FLA_SPDinv_s fla_spdinv_t;



#define FLA_SUBPROBLEM              0

#define FLA_UNBLOCKED_VARIANT1    (-1)
#define FLA_UNBLOCKED_VARIANT2    (-2)
#define FLA_UNBLOCKED_VARIANT3    (-3)
#define FLA_UNBLOCKED_VARIANT4    (-4)

#define FLA_BLOCKED_VARIANT1        1
#define FLA_BLOCKED_VARIANT2        2
#define FLA_BLOCKED_VARIANT3        3
#define FLA_BLOCKED_VARIANT4        4
#define FLA_BLOCKED_VARIANT5        5
#define FLA_BLOCKED_VARIANT6        6
#define FLA_BLOCKED_VARIANT7        7
#define FLA_BLOCKED_VARIANT8        8
#define FLA_BLOCKED_VARIANT9        9
#define FLA_BLOCKED_VARIANT10       10
#define FLA_BLOCKED_VARIANT11       11
#define FLA_BLOCKED_VARIANT12       12
#define FLA_BLOCKED_VARIANT13       13
#define FLA_BLOCKED_VARIANT14       14
#define FLA_BLOCKED_VARIANT15       15
#define FLA_BLOCKED_VARIANT16       16
#define FLA_BLOCKED_VARIANT17       17
#define FLA_BLOCKED_VARIANT18       18
#define FLA_BLOCKED_VARIANT19       19
#define FLA_BLOCKED_VARIANT20       20

#define FLA_Cntl_matrix_type( cntl )  cntl->matrix_type
#define FLA_Cntl_blocksize( cntl )    cntl->blocksize
#define FLA_Cntl_variant( cntl )      cntl->variant

#define FLA_Cntl_sub_gemm( cntl )     cntl->sub_gemm
#define FLA_Cntl_sub_gemm1( cntl )    cntl->sub_gemm1
#define FLA_Cntl_sub_gemm2( cntl )    cntl->sub_gemm2
#define FLA_Cntl_sub_gemm3( cntl )    cntl->sub_gemm3
#define FLA_Cntl_sub_gemm4( cntl )    cntl->sub_gemm4
#define FLA_Cntl_sub_gemm5( cntl )    cntl->sub_gemm5
#define FLA_Cntl_sub_gemm6( cntl )    cntl->sub_gemm6
#define FLA_Cntl_sub_gemm7( cntl )    cntl->sub_gemm7
#define FLA_Cntl_sub_gemm8( cntl )    cntl->sub_gemm8
#define FLA_Cntl_sub_hemm( cntl )     cntl->sub_hemm
#define FLA_Cntl_sub_herk( cntl )     cntl->sub_herk
#define FLA_Cntl_sub_her2k( cntl )    cntl->sub_her2k
#define FLA_Cntl_sub_symm( cntl )     cntl->sub_symm
#define FLA_Cntl_sub_syrk( cntl )     cntl->sub_syrk
#define FLA_Cntl_sub_syr2k( cntl )    cntl->sub_syr2k
#define FLA_Cntl_sub_trmm( cntl )     cntl->sub_trmm
#define FLA_Cntl_sub_trsm( cntl )     cntl->sub_trsm
#define FLA_Cntl_sub_trsm1( cntl )    cntl->sub_trsm1
#define FLA_Cntl_sub_trsm2( cntl )    cntl->sub_trsm2
#define FLA_Cntl_sub_trsm3( cntl )    cntl->sub_trsm3
#define FLA_Cntl_sub_trsm4( cntl )    cntl->sub_trsm4
#define FLA_Cntl_sub_chol( cntl )     cntl->sub_chol
#define FLA_Cntl_sub_lu( cntl )       cntl->sub_lu
#define FLA_Cntl_sub_qr( cntl )       cntl->sub_qr
#define FLA_Cntl_sub_lq( cntl )       cntl->sub_lq
#define FLA_Cntl_sub_trinv( cntl )    cntl->sub_trinv
#define FLA_Cntl_sub_hess( cntl )     cntl->sub_hess
#define FLA_Cntl_sub_ttmm( cntl )     cntl->sub_ttmm
#define FLA_Cntl_sub_sylv1( cntl )    cntl->sub_sylv1
#define FLA_Cntl_sub_sylv2( cntl )    cntl->sub_sylv2
#define FLA_Cntl_sub_sylv3( cntl )    cntl->sub_sylv3
#define FLA_Cntl_sub_swap( cntl )     cntl->sub_swap
#define FLA_Cntl_sub_trans( cntl )    cntl->sub_trans


fla_gemm_t*  FLA_Cntl_gemm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_gemm_t* sub_gemm );
fla_symm_t*  FLA_Cntl_symm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_symm_t* sub_symm, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 );
fla_hemm_t*  FLA_Cntl_hemm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_hemm_t* sub_hemm, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 );
fla_herk_t*  FLA_Cntl_herk_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_herk_t* sub_herk, fla_gemm_t* sub_gemm );
fla_her2k_t* FLA_Cntl_her2k_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_her2k_t* sub_her2k, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 );
fla_syrk_t*  FLA_Cntl_syrk_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_syrk_t* sub_syrk, fla_gemm_t* sub_gemm );
fla_syr2k_t* FLA_Cntl_syr2k_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_syr2k_t* sub_syr2k, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 );
fla_trmm_t*  FLA_Cntl_trmm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_trmm_t* sub_trmm, fla_gemm_t* sub_gemm );
fla_trsm_t*  FLA_Cntl_trsm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_trsm_t* sub_trsm, fla_gemm_t* sub_gemm );
fla_chol_t*  FLA_Cntl_chol_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_chol_t* sub_chol, fla_syrk_t* sub_syrk, fla_herk_t* sub_herk, fla_trsm_t* sub_trsm, fla_gemm_t* sub_gemm );
fla_lu_t*    FLA_Cntl_lu_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_lu_t* sub_lu, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2, fla_gemm_t* sub_gemm3, fla_trsm_t* sub_trsm1, fla_trsm_t* sub_trsm2 );
fla_qr_t*    FLA_Cntl_qr_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_qr_t* sub_qr, fla_trmm_t* sub_trmm1, fla_trmm_t* sub_trmm2, fla_trmm_t* sub_trmm3, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 );
fla_lq_t*    FLA_Cntl_lq_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_lq_t* sub_lq, fla_trmm_t* sub_trmm1, fla_trmm_t* sub_trmm2, fla_trmm_t* sub_trmm3, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 );
fla_trinv_t* FLA_Cntl_trinv_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_trinv_t* sub_trinv, fla_trmm_t* sub_trmm, fla_trsm_t* sub_trsm1, fla_trsm_t* sub_trsm2, fla_gemm_t* sub_gemm );
fla_hess_t*  FLA_Cntl_hess_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_hess_t* sub_hess, fla_trmm_t* sub_trmm1, fla_trmm_t* sub_trmm2, fla_trmm_t* sub_trmm3, fla_trmm_t* sub_trmm4, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2, fla_gemm_t* sub_gemm3 );
fla_ttmm_t*  FLA_Cntl_ttmm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_ttmm_t* sub_ttmm, fla_syrk_t* sub_syrk, fla_herk_t* sub_herk, fla_trmm_t* sub_trmm, fla_gemm_t* sub_gemm );
fla_sylv_t*  FLA_Cntl_sylv_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_sylv_t* sub_sylv1, fla_sylv_t* sub_sylv2, fla_sylv_t* sub_sylv3, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2, fla_gemm_t* sub_gemm3, fla_gemm_t* sub_gemm4, fla_gemm_t* sub_gemm5, fla_gemm_t* sub_gemm6, fla_gemm_t* sub_gemm7, fla_gemm_t* sub_gemm8 );
fla_transpose_t*  FLA_Cntl_transpose_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_transpose_t* sub_trans, fla_swap_t* sub_swap );
fla_swap_t*  FLA_Cntl_swap_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_swap_t* sub_swap );
fla_spdinv_t* FLA_Cntl_spdinv_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_chol_t* sub_chol, fla_trinv_t* sub_trinv, fla_ttmm_t* sub_ttmm );
void         FLA_Cntl_obj_free( void* cntl );

