/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLA_type_defs.h"

// --- Pointer-accessing FLAME macro definitions ------------------------------------

#ifdef FLA_ENABLE_PLAPACK2E
  
  #define FLA_INT_PTR( x ) \
    ( ((x).base)->templ == NULL ? \
      FLA_INT_PTR_OLD( x ) : PLA_INT_PTR( x ) )

  #define FLA_FLOAT_PTR( x ) \
    ( ((x).base)->templ == NULL ? \
      FLA_FLOAT_PTR_OLD( x ) : PLA_FLOAT_PTR( x ) )

  #define FLA_DOUBLE_PTR( x ) \
    ( ((x).base)->templ == NULL ? \
      FLA_DOUBLE_PTR_OLD( x ) : PLA_DOUBLE_PTR( x ) )

  #define FLA_COMPLEX_PTR( x ) \
    ( ((x).base)->templ == NULL ? \
      FLA_COMPLEX_PTR_OLD( x ) : PLA_COMPLEX_PTR( x ) )

  #define FLA_DOUBLE_COMPLEX_PTR( x ) \
    ( ((x).base)->templ == NULL ? \
      FLA_DOUBLE_COMPLEX_PTR_OLD( x ) : PLA_DOUBLE_COMPLEX_PTR( x ) )

  #include "PLA_macro_ptr_defs.h"

#else
  
  #define FLA_INT_PTR             FLA_INT_PTR_OLD
  #define FLA_FLOAT_PTR           FLA_FLOAT_PTR_OLD
  #define FLA_DOUBLE_PTR          FLA_DOUBLE_PTR_OLD
  #define FLA_COMPLEX_PTR         FLA_COMPLEX_PTR_OLD
  #define FLA_DOUBLE_COMPLEX_PTR  FLA_DOUBLE_COMPLEX_PTR_OLD

  #include "PLA_FLA_external.h"

#endif

#define FLA_CONSTANT_I_OFFSET  0
#define FLA_CONSTANT_S_OFFSET  ( sizeof(double) )
#define FLA_CONSTANT_D_OFFSET  ( sizeof(double) + sizeof(double) )
#define FLA_CONSTANT_C_OFFSET  ( sizeof(double) + sizeof(double) + sizeof(double) )
#define FLA_CONSTANT_Z_OFFSET  ( sizeof(double) + sizeof(double) + sizeof(double) + sizeof( scomplex ) )

#define FLA_INT_PTR_OLD( x ) \
  ( ((x).base)->datatype == FLA_CONSTANT ? \
    ( int * )      ( ( ( char * )     ((x).base)->buffer ) + FLA_CONSTANT_I_OFFSET             ) : \
                   ( ( ( int * )      ((x).base)->buffer ) + ( size_t ) (x).offn * ((x).base)->ldim + (x).offm ) )

#define FLA_FLOAT_PTR_OLD( x ) \
  ( ((x).base)->datatype == FLA_CONSTANT ? \
    ( float * )    ( ( ( char * )     ((x).base)->buffer ) + FLA_CONSTANT_S_OFFSET             ) : \
                   ( ( ( float * )    ((x).base)->buffer ) + ( size_t ) (x).offn * ((x).base)->ldim + (x).offm ) )

#define FLA_DOUBLE_PTR_OLD( x ) \
  ( ((x).base)->datatype == FLA_CONSTANT ? \
    ( double * )   ( ( ( char * )     ((x).base)->buffer ) + FLA_CONSTANT_D_OFFSET             ) : \
                   ( ( ( double * )   ((x).base)->buffer ) + ( size_t ) (x).offn * ((x).base)->ldim + (x).offm ) )

#define FLA_COMPLEX_PTR_OLD( x ) \
  ( ((x).base)->datatype == FLA_CONSTANT ? \
    ( scomplex * ) ( ( ( char * )     ((x).base)->buffer ) + FLA_CONSTANT_C_OFFSET             ) : \
                   ( ( ( scomplex * ) ((x).base)->buffer ) + ( size_t ) (x).offn * ((x).base)->ldim + (x).offm ) )

#define FLA_DOUBLE_COMPLEX_PTR_OLD( x ) \
  ( ((x).base)->datatype == FLA_CONSTANT ? \
    ( dcomplex * ) ( ( ( char * )     ((x).base)->buffer ) + FLA_CONSTANT_Z_OFFSET             ) : \
                   ( ( ( dcomplex * ) ((x).base)->buffer ) + ( size_t ) (x).offn * ((x).base)->ldim + (x).offm ) )
