/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

// -----------------------------------------------------------------------------

fla_blocksize_t* FLA_Blocksize_create( int b_s, int b_d, int b_c, int b_z );
fla_blocksize_t* FLA_Blocksize_create_copy( fla_blocksize_t* bp );
void             FLA_Blocksize_set( fla_blocksize_t* bp, int b_s, int b_d, int b_c, int b_z );
void             FLA_Blocksize_scale( fla_blocksize_t* bp, double factor );
void             FLA_Blocksize_free( fla_blocksize_t* bp );
int              FLA_Blocksize_extract( FLA_Datatype dt, fla_blocksize_t* bp );

fla_blocksize_t* FLA_Query_blocksizes( FLA_Dimension dim );
int              FLA_Query_blocksize( FLA_Datatype dt, FLA_Dimension dim );

int              FLA_Determine_blocksize( FLA_Obj A_unproc, FLA_Quadrant to_dir, fla_blocksize_t* cntl_blocksizes );
int              FLA_determine_matrix_size( FLA_Obj A_unproc, FLA_Quadrant to_dir );
F_INT            FLA_F2C( fla_query_blocksize_f ) ( F_INT *dt, F_INT *dim );


// -----------------------------------------------------------------------------

FLA_Bool      FLA_Check_parameters( void );
FLA_Bool      FLA_Check_parameters_set( FLA_Bool new_val );
void          FLA_Check_error_code_helper( int code, char* file, int line );
FLA_Error     FLA_Check_valid_side( FLA_Side side );
FLA_Error     FLA_Check_valid_uplo( FLA_Uplo uplo );
FLA_Error     FLA_Check_valid_trans( FLA_Trans trans );
FLA_Error     FLA_Check_valid_diag( FLA_Diag diag );
FLA_Error     FLA_Check_valid_conj( FLA_Conj conj );
FLA_Error     FLA_Check_valid_direct( FLA_Conj direct );
FLA_Error     FLA_Check_valid_storev( FLA_Conj storev );
FLA_Error     FLA_Check_valid_datatype( FLA_Datatype datatype );
FLA_Error     FLA_Check_valid_object_datatype( FLA_Obj A );
FLA_Error     FLA_Check_floating_datatype( FLA_Datatype datatype );
FLA_Error     FLA_Check_int_datatype( FLA_Datatype datatype );
FLA_Error     FLA_Check_real_datatype( FLA_Datatype datatype );
FLA_Error     FLA_Check_complex_datatype( FLA_Datatype datatype );
FLA_Error     FLA_Check_floating_object( FLA_Obj A );
FLA_Error     FLA_Check_int_object( FLA_Obj A );
FLA_Error     FLA_Check_real_object( FLA_Obj A );
FLA_Error     FLA_Check_complex_object( FLA_Obj A );
FLA_Error     FLA_Check_consistent_datatype( FLA_Datatype datatype, FLA_Obj A );
FLA_Error     FLA_Check_consistent_object_datatype( FLA_Obj A, FLA_Obj B );
FLA_Error     FLA_Check_identical_object_precision( FLA_Obj A, FLA_Obj B );
FLA_Error     FLA_Check_square( FLA_Obj A );
FLA_Error     FLA_Check_if_scalar( FLA_Obj A );
FLA_Error     FLA_Check_if_vector( FLA_Obj A );
FLA_Error     FLA_Check_conformal_dims( FLA_Trans trans, FLA_Obj A, FLA_Obj B );
FLA_Error     FLA_Check_matrix_matrix_dims( FLA_Trans transa, FLA_Trans transb, FLA_Obj A, FLA_Obj B, FLA_Obj C );
FLA_Error     FLA_Check_matrix_vector_dims( FLA_Trans trans, FLA_Obj A, FLA_Obj x, FLA_Obj y );
FLA_Error     FLA_Check_equal_vector_lengths( FLA_Obj x, FLA_Obj y );
FLA_Error     FLA_Check_conj_trans_and_datatype( FLA_Trans trans, FLA_Obj A );
FLA_Error     FLA_Check_hess_indices( FLA_Obj A, int ilo, int ihi );
FLA_Error     FLA_Check_vector_length( FLA_Obj x, int expected_length );
FLA_Error     FLA_Check_null_pointer( void* ptr );
FLA_Error     FLA_Check_object_dims( FLA_Trans trans, int m, int n, FLA_Obj A );
FLA_Error     FLA_Check_valid_pivot_type( FLA_Pivot_type ptype );
FLA_Error     FLA_Check_block_householder_transform( FLA_Store storev, FLA_Obj A, FLA_Obj S );
FLA_Error     FLA_Check_malloc_pointer( void* ptr );
FLA_Error     FLA_Check_base_buffer_mismatch( FLA_Obj A, FLA_Obj B );
FLA_Error     FLA_Check_adjacent_objects_2x2( FLA_Obj A11, FLA_Obj A12,
                                              FLA_Obj A21, FLA_Obj A22 );
FLA_Error     FLA_Check_adjacent_objects_2x1( FLA_Obj AT,
                                              FLA_Obj AB );
FLA_Error     FLA_Check_adjacent_objects_1x2( FLA_Obj AL, FLA_Obj AR );
FLA_Error     FLA_Check_blocksize_value( int b );
FLA_Error     FLA_Check_blocksize_object( FLA_Datatype datatype, fla_blocksize_t* bp );
FLA_Error     FLA_Check_file_descriptor( int fd );
FLA_Error     FLA_Check_lseek_result( int requested_offset, int lseek_r_val );
FLA_Error     FLA_Check_close_result( int close_r_val );
FLA_Error     FLA_Check_unlink_result( int unlink_r_val );
FLA_Error     FLA_Check_read_result( int requested_size, int read_r_val );
FLA_Error     FLA_Check_write_result( int requested_size, int write_r_val );
FLA_Error     FLA_Check_valid_quadrant( FLA_Quadrant quad );
FLA_Error     FLA_Check_pivot_vector_length( FLA_Obj ipiv );
FLA_Error     FLA_Check_vector_length_min( FLA_Obj x, int min_length );
FLA_Error     FLA_Check_pthread_create_result( int pthread_create_r_val );
FLA_Error     FLA_Check_pthread_join_result( int pthread_join_r_val );
FLA_Error     FLA_Check_valid_isgn_value( FLA_Obj isgn );
FLA_Error     FLA_Check_sylv_matrix_dims( FLA_Obj A, FLA_Obj B, FLA_Obj C );
FLA_Error     FLA_Check_chol_failure( FLA_Error r_val );
FLA_Error     FLA_Check_valid_elemtype( FLA_Elemtype elemtype );
FLA_Error     FLA_Check_posix_memalign_failure( int r_val );
FLA_Error     FLA_Check_submatrix_dims_and_offset( int m, int n, int i, int j, FLA_Obj A );
FLA_Error     FLA_Check_object_scalar_elemtype( FLA_Obj A );
FLA_Error     FLA_Check_object_matrix_elemtype( FLA_Obj A );
FLA_Error     FLA_Check_num_threads( unsigned int n_threads );
FLA_Error     FLA_Check_conj_and_datatype( FLA_Conj conj, FLA_Obj A );
FLA_Error     FLA_Check_valid_complex_trans( FLA_Trans trans );
FLA_Error     FLA_Check_valid_real_trans( FLA_Trans trans );
FLA_Error     FLA_Check_valid_blas_trans( FLA_Trans trans );
FLA_Error     FLA_Check_nonconstant_datatype( FLA_Datatype datatype );
FLA_Error     FLA_Check_nonconstant_object( FLA_Obj A );
FLA_Error     FLA_Check_m_against_ldim( int m, int ldim );
FLA_Error     FLA_Check_identical_object_datatype( FLA_Obj A, FLA_Obj B );
FLA_Error     FLA_Check_divide_by_zero( FLA_Obj alpha );
FLA_Error     FLA_Check_identical_object_elemtype( FLA_Obj A, FLA_Obj B );
FLA_Error     FLA_Check_pivot_index_range( FLA_Obj p, int k1, int k2 );



// -----------------------------------------------------------------------------

char*         FLA_Error_string_for_code( int code );
void          FLA_Error_messages_init( void );
void          FLA_Print_message( char *str, char *file, int line );
void          FLA_Abort( void );

void          FLA_F2C( fla_abort_f )( void );



// -----------------------------------------------------------------------------

FLA_Error     FLA_Obj_OOC_create( FLA_Datatype datatype, int m, int n, off_t offset, int fd, FLA_Obj *obj );
FLA_Error     FLA_Obj_OOC_to_INC( FLA_Obj obj_ooc, FLA_Obj obj_inc );
FLA_Error     FLA_Obj_INC_to_OOC( FLA_Obj obj_inc, FLA_Obj obj_ooc );
FLA_Error     FLA_File_create( char* filename, FLA_Datatype datatype, size_t size, int* fd );
FLA_Error     FLA_File_open( char *filename, int *fd );
FLA_Error     FLA_File_close( int fd );
FLA_Error     FLA_File_unlink( char *filename );
FLA_Error     FLA_Read( size_t size, int fd, off_t offset, void *buffer );
FLA_Error     FLA_Write( size_t size, void *buffer, int fd, off_t offset );
FLA_Error     FLA_Read_matrix( int typesize, int length, int width, int fd,
                               int buffer_from, int ldim_from,
                               char *buffer_to, int ldim_to );
FLA_Error     FLA_Write_matrix( int typesize, int length, int width, int fd,
                                char *buffer_from, int ldim_from,
                                int buffer_to, int ldim_to );



// -----------------------------------------------------------------------------

FLA_Bool      FLA_Initialized( void );
void          FLA_Init( void );
void          FLA_Init_constants( void );
void          FLA_Init_memcounter( void );
void          FLA_Init_temp( void );
void          FLA_Finalize( void );
void          FLA_Finalize_constants( void );
void          FLA_Finalize_memcounter( void );
void          FLA_Finalize_temp( void );

F_INT         FLA_F2C( fla_initialized_f )( void );
void          FLA_F2C( fla_init_fortran )( FLA_Obj* minus_one, FLA_Obj* zero, FLA_Obj* one, FLA_Obj* two );
void          FLA_F2C( fla_finalize_fortran )( void );
void          FLA_F2C( fla_init_f )( void );
void          FLA_F2C( fla_finalize_f )( void );



//------------------------------------------------------------------------------

void          FLA_Lock_init( FLA_Lock* fla_lock_ptr );
void          FLA_Lock_destroy( FLA_Lock* fla_lock_ptr );
void          FLA_Lock_acquire( FLA_Lock* fla_lock_ptr );
void          FLA_Lock_release( FLA_Lock* fla_lock_ptr );



// -----------------------------------------------------------------------------

void*         FLA_malloc( size_t size );
void*         FLA_realloc( void* old_ptr, size_t size );
void          FLA_free( void *ptr );
 


// -----------------------------------------------------------------------------

FLA_Bool      FLA_Obj_is_int( FLA_Obj A );
FLA_Bool      FLA_Obj_is_floating_point( FLA_Obj A );
FLA_Bool      FLA_Obj_is_constant( FLA_Obj A );
FLA_Bool      FLA_Obj_is_real( FLA_Obj A );
FLA_Bool      FLA_Obj_is_complex( FLA_Obj A );
FLA_Bool      FLA_Obj_is_single_precision( FLA_Obj A );
FLA_Bool      FLA_Obj_is_double_precision( FLA_Obj A );
FLA_Bool      FLA_Obj_is_scalar( FLA_Obj A );
FLA_Bool      FLA_Obj_is_vector( FLA_Obj A );
FLA_Bool      FLA_Obj_has_zero_dim( FLA_Obj A );
FLA_Bool      FLA_Obj_is_conformal_to( FLA_Trans trans, FLA_Obj A, FLA_Obj B );
FLA_Bool      FLA_Obj_is( FLA_Obj A, FLA_Obj B );
FLA_Bool      FLA_Obj_equals( FLA_Obj A, FLA_Obj B );
FLA_Error     FLA_Obj_set_to_scalar( FLA_Obj alpha, FLA_Obj A );
FLA_Error     FLA_Obj_set_diagonal_to_scalar( FLA_Obj alpha, FLA_Obj A );
FLA_Error     FLA_Obj_set_to_identity( FLA_Obj A );
FLA_Error     FLA_Obj_add_to_diagonal( void *diag_value, FLA_Obj A );
FLA_Error     FLA_Obj_shift_diagonal( FLA_Obj sigma, FLA_Obj A );
FLA_Error     FLA_Obj_show( char *s1, FLA_Obj A, char *format, char *s2 );

FLA_Error     FLA_Copy_submatrix_to_global( FLA_Trans trans, int m, int n, void* buffer, int ldim, int i, int j, FLA_Obj obj );
FLA_Error     FLA_Copy_global_to_submatrix( FLA_Trans trans, int i, int j, FLA_Obj obj, int m, int n, void* buffer, int ldim );
FLA_Error     FLA_Axpy_submatrix_to_global( FLA_Trans trans, FLA_Obj alpha, int m, int n, void* buffer, int ldim, int i, int j, FLA_Obj C );
FLA_Error     FLA_Axpy_global_to_submatrix( FLA_Trans trans, FLA_Obj alpha, int i, int j, FLA_Obj C, int m, int n, void* buffer, int ldim );

void          FLA_F2C( fla_obj_is_f                     ) ( F_INT *A, F_INT *B, F_INT *IERROR );
void          FLA_F2C( fla_obj_equals_f                 ) ( F_INT *A, F_INT *B, F_INT *IERROR );
void          FLA_F2C( fla_obj_set_to_scalar_f          ) ( F_INT *alpha, F_INT *A, F_INT *IERROR );
void          FLA_F2C( fla_obj_set_diagonal_to_scalar_f ) ( F_INT *alpha, F_INT* A, F_INT *IERROR );
void          FLA_F2C( fla_obj_set_to_identity_f        ) ( F_INT *A, F_INT *IERROR );
void          FLA_F2C( fla_obj_add_to_diagonal_f        ) ( F_INT *value, F_INT *A, F_INT *IERROR );
void          FLA_F2C( fla_obj_shift_diagonal_f         ) ( F_INT *sigma, F_INT *A, F_INT *IERROR );
void          FLA_F2C( fla_obj_show_f                   ) ( F_INT *s1, F_INT *A, F_INT *format, F_INT *s2, F_INT *IERROR );

FLA_Error     FLA_Obj_equals_check( FLA_Obj A, FLA_Obj B );
FLA_Error     FLA_Obj_set_to_scalar_check( FLA_Obj alpha, FLA_Obj A );
FLA_Error     FLA_Obj_set_diagonal_to_scalar_check( FLA_Obj alpha, FLA_Obj A );
FLA_Error     FLA_Obj_set_to_identity_check( FLA_Obj A );
FLA_Error     FLA_Obj_add_to_diagonal_check( void *diag_value, FLA_Obj A );
FLA_Error     FLA_Obj_shift_diagonal_check( FLA_Obj sigma, FLA_Obj A );
FLA_Error     FLA_Obj_scale_diagonal_check( FLA_Obj alpha, FLA_Obj A );

FLA_Error     FLA_Copy_submatrix_to_global_check( FLA_Trans trans, int m, int n, void* buffer, int ldim, int i, int j, FLA_Obj obj );
FLA_Error     FLA_Copy_global_to_submatrix_check( FLA_Trans trans, int i, int j, FLA_Obj obj, int m, int n, void* buffer, int ldim );
FLA_Error     FLA_Axpy_submatrix_to_global_check( FLA_Trans trans, FLA_Obj alpha, int m, int n, void* buffer, int ldim, int i, int j, FLA_Obj C );
FLA_Error     FLA_Axpy_global_to_submatrix_check( FLA_Trans trans, FLA_Obj alpha, int i, int j, FLA_Obj C, int m, int n, void* buffer, int ldim );



// -----------------------------------------------------------------------------

FLA_Error     FLA_Obj_create( FLA_Datatype datatype, int m, int n, FLA_Obj *obj );
FLA_Error     FLA_Obj_create_ext( FLA_Datatype datatype, FLA_Elemtype elemtype, int m, int n, int m_global, int n_global, FLA_Obj *obj );
FLA_Error     FLA_Obj_create_conf_to( FLA_Trans trans, FLA_Obj old, FLA_Obj *obj );
FLA_Error     FLA_Obj_create_without_buffer( FLA_Datatype datatype, int m, int n, FLA_Obj *obj );
FLA_Error     FLA_Obj_constant_create( double const_real, FLA_Obj *obj );
FLA_Error     FLA_Obj_complex_constant_create( double const_real, double const_imag, FLA_Obj *obj );
FLA_Error     FLA_Obj_attach_buffer( void *buffer, int lda, FLA_Obj *obj );
FLA_Error     FLA_Obj_free( FLA_Obj *obj );
FLA_Error     FLA_Obj_free_without_buffer( FLA_Obj *obj );
FLA_Datatype  FLA_Obj_datatype( FLA_Obj obj );
FLA_Elemtype  FLA_Obj_elemtype( FLA_Obj obj );
int           FLA_Obj_datatype_size( FLA_Datatype datatype );
int           FLA_Obj_elem_size( FLA_Obj obj );
int           FLA_Obj_length( FLA_Obj obj );
int           FLA_Obj_width( FLA_Obj obj );
int           FLA_Obj_vector_dim( FLA_Obj obj );
int           FLA_Obj_min_dim( FLA_Obj obj );
int           FLA_Obj_max_dim( FLA_Obj obj );
int           FLA_Obj_ldim( FLA_Obj obj );
void*         FLA_Obj_buffer( FLA_Obj obj );

void          FLA_F2C( fla_obj_create_f                  ) ( F_INT *datatype, F_INT *m, F_INT *n, F_INT *obj, F_INT *IERROR );
void          FLA_F2C( fla_obj_create_conf_to_f          ) ( F_INT *trans, F_INT *old, F_INT *obj, F_INT *IERROR );
void          FLA_F2C( fla_obj_create_w_out_buff_f       ) ( F_INT *datatype, F_INT *m, F_INT *n, F_INT *obj, F_INT *IERROR );
void          FLA_F2C( fla_obj_constant_create_f         ) ( F_DOUBLE* const_real, F_INT* obj, F_INT *IERROR );
void          FLA_F2C( fla_obj_complex_constant_create_f ) ( F_DOUBLE* const_real, F_DOUBLE* const_imag, F_INT* obj, F_INT *IERROR );
void          FLA_F2C( fla_obj_attach_buff_f             ) ( F_INT *buffer, F_INT *lda, F_INT *obj, F_INT *IERROR );
void          FLA_F2C( fla_obj_free_f                    ) ( F_INT *obj, F_INT *IERROR );
void          FLA_F2C( fla_obj_free_w_out_buff_f         ) ( F_INT *obj, F_INT *IERROR );
F_INT         FLA_F2C( fla_obj_datatype_f                ) ( F_INT *obj );
F_INT         FLA_F2C( fla_obj_datatype_size_f           ) ( F_INT *datatype );
F_INT         FLA_F2C( fla_obj_elem_size_f               ) ( F_INT *obj );
F_INT         FLA_F2C( fla_obj_length_f                  ) ( F_INT *obj );
F_INT         FLA_F2C( fla_obj_width_f                   ) ( F_INT *obj );
F_INT         FLA_F2C( fla_obj_vector_dim_f              ) ( F_INT *obj );
F_INT         FLA_F2C( fla_obj_min_dim_f                 ) ( F_INT *obj );
F_INT         FLA_F2C( fla_obj_max_dim_f                 ) ( F_INT *obj );
F_INT         FLA_F2C( fla_obj_ldim_f                    ) ( F_INT *obj );



// -----------------------------------------------------------------------------

void          FLA_Param_map_to_blas_trans( FLA_Trans trans, void* blas_trans );
void          FLA_Param_map_to_blas_uplo( FLA_Uplo uplo, void* blas_uplo );
void          FLA_Param_map_to_blas_side( FLA_Uplo side, void* blas_side );
void          FLA_Param_map_to_blas_diag( FLA_Diag diag, void* blas_diag );
void          FLA_Param_map_to_lapack_direct( FLA_Direct direct, void* lapack_direct );
void          FLA_Param_map_to_lapack_storev( FLA_Store storev, void* lapack_storev );



// -----------------------------------------------------------------------------

FLA_Error     FLA_Part_2x2( FLA_Obj A,  FLA_Obj *A11, FLA_Obj *A12,
                                        FLA_Obj *A21, FLA_Obj *A22,
                            int    mb,  int       nb, FLA_Quadrant quadrant );

FLA_Error     FLA_Merge_2x2( FLA_Obj A11, FLA_Obj A12,
                             FLA_Obj A21, FLA_Obj A22,  FLA_Obj *A );
 
FLA_Error     FLA_Repart_2x2_to_3x3( FLA_Obj ATL, FLA_Obj ATR,  FLA_Obj *A00, FLA_Obj *A01, FLA_Obj *A02,
                                                                FLA_Obj *A10, FLA_Obj *A11, FLA_Obj *A12,
                                     FLA_Obj ABL, FLA_Obj ABR,  FLA_Obj *A20, FLA_Obj *A21, FLA_Obj *A22,
                                     int     mb,  int      nb,  FLA_Quadrant quadrant );

FLA_Error     FLA_Cont_with_3x3_to_2x2( FLA_Obj *ATL, FLA_Obj *ATR,  FLA_Obj A00, FLA_Obj A01, FLA_Obj A02,
                                                                     FLA_Obj A10, FLA_Obj A11, FLA_Obj A12,
                                        FLA_Obj *ABL, FLA_Obj *ABR,  FLA_Obj A20, FLA_Obj A21, FLA_Obj A22,
                                                                     FLA_Quadrant quadrant );

FLA_Error     FLA_Part_2x1 ( FLA_Obj A,  FLA_Obj *A1,
                                         FLA_Obj *A2,
                             int    mb,  FLA_Side side );

FLA_Error     FLA_Merge_2x1( FLA_Obj AT,
                             FLA_Obj AB,  FLA_Obj *A );

FLA_Error     FLA_Repart_2x1_to_3x1( FLA_Obj AT,  FLA_Obj *A0,
                                                  FLA_Obj *A1,
                                     FLA_Obj AB,  FLA_Obj *A2,
                                     int     mb,  FLA_Side side );

FLA_Error     FLA_Cont_with_3x1_to_2x1( FLA_Obj *AT,  FLA_Obj A0,
                                                      FLA_Obj A1,
                                        FLA_Obj *AB,  FLA_Obj A2,
                                                      FLA_Side side );

FLA_Error     FLA_Part_1x2( FLA_Obj A,  FLA_Obj *A1, FLA_Obj *A2,
                                        int      nb, FLA_Side side );
 
FLA_Error     FLA_Merge_1x2( FLA_Obj AL, FLA_Obj AR,  FLA_Obj *A );

FLA_Error     FLA_Repart_1x2_to_1x3( FLA_Obj  AL,              FLA_Obj  AR,
                                     FLA_Obj *A0, FLA_Obj *A1, FLA_Obj *A2,
                                                  int      nb, FLA_Side side );

FLA_Error     FLA_Cont_with_1x3_to_1x2( FLA_Obj *AL,              FLA_Obj *AR,
                                        FLA_Obj  A0, FLA_Obj  A1, FLA_Obj  A2,
                                                                  FLA_Side side );


void          FLA_F2C( fla_part_2x2_f )
              ( F_INT *A,   F_INT *A11, F_INT *A12,
                            F_INT *A21, F_INT *A22, 
                F_INT *mb,  F_INT *nb,  F_INT *quadrant, F_INT *IERROR );

void          FLA_F2C( fla_merge_2x2_f )
              ( F_INT *A11, F_INT *A12,
                F_INT *A21, F_INT *A22,  F_INT *A,  F_INT *IERROR );

void          FLA_F2C( fla_repart_2x2_to_3x3_f )
              ( F_INT *ATL, F_INT *ATR,  F_INT *A00, F_INT *A01, F_INT *A02,
                                         F_INT *A10, F_INT *A11, F_INT *A12,
                F_INT *ABL, F_INT *ABR,  F_INT *A20, F_INT *A21, F_INT *A22,
                F_INT *mb,  F_INT *nb,   F_INT *quadrant, F_INT *IERROR );

void          FLA_F2C( fla_cont_with_3x3_to_2x2_f )
              ( F_INT *ATL, F_INT *ATR,  F_INT *A00, F_INT *A01, F_INT *A02,
                                         F_INT *A10, F_INT *A11, F_INT *A12,
                F_INT *ABL, F_INT *ABR,  F_INT *A20, F_INT *A21, F_INT *A22,
                                         F_INT *quadrant, F_INT *IERROR );

void          FLA_F2C( fla_part_2x1_f )
              ( F_INT *A,   F_INT *A1,
                            F_INT *A2,
                F_INT *mb,  F_INT *side, F_INT *IERROR );

void          FLA_F2C( fla_merge_2x1_f )
              ( F_INT *AT,
                F_INT *AB,  F_INT *A,  F_INT *IERROR );

void          FLA_F2C( fla_repart_2x1_to_3x1_f )
              ( F_INT *AT,  F_INT *A0,
                            F_INT *A1,
                F_INT *AB,  F_INT *A2,
                F_INT *mb,  F_INT *side, F_INT *IERROR );
 
void          FLA_F2C( fla_cont_with_3x1_to_2x1_f )
              ( F_INT *AT,  F_INT *A0,
                            F_INT *A1,
                F_INT *AB,  F_INT *A2,
                            F_INT *side, F_INT *IERROR );

void          FLA_F2C( fla_part_1x2_f )
              ( F_INT *A,  F_INT *A1, F_INT *A2,
                           F_INT *nb, F_INT *side,  F_INT *IERROR );
 
void          FLA_F2C( fla_merge_1x2_f )
              ( F_INT *AL, F_INT *AR,  F_INT *A,  F_INT *IERROR );

void          FLA_F2C( fla_repart_1x2_to_1x3_f )
              ( F_INT *AL,            F_INT *AR,
                F_INT *A0, F_INT *A1, F_INT *A2,
                           F_INT *nb, F_INT *side,  F_INT *IERROR );

void          FLA_F2C( fla_cont_with_1x3_to_1x2_f )
              ( F_INT *AL,            F_INT *AR,
                F_INT *A0, F_INT *A1, F_INT *A2,
                                      F_INT *side,  F_INT *IERROR );

