/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"


/* ***************************************************************************

   FLA_Obj_is_int( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_int( FLA_Obj A )
{
  FLA_Datatype datatype;
  FLA_Bool     r_val;

  datatype = FLA_Obj_datatype( A );

  if ( datatype == FLA_INT )
    r_val = TRUE;
  else
    r_val = FALSE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is_floating_point( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_floating_point( FLA_Obj A )
{
  FLA_Datatype datatype;
  FLA_Bool     r_val;

  datatype = FLA_Obj_datatype( A );

  if ( datatype == FLA_FLOAT || datatype == FLA_COMPLEX ||
       datatype == FLA_DOUBLE || datatype == FLA_DOUBLE_COMPLEX )
    r_val = TRUE;
  else
    r_val = FALSE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is_constant( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_constant( FLA_Obj A )
{
  FLA_Datatype datatype;
  FLA_Bool     r_val;

  datatype = FLA_Obj_datatype( A );

  if ( datatype == FLA_CONSTANT )
    r_val = TRUE;
  else
    r_val = FALSE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is_real( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_real( FLA_Obj A )
{
  FLA_Datatype datatype;
  FLA_Bool     r_val;

  datatype = FLA_Obj_datatype( A );

  if ( datatype == FLA_CONSTANT || datatype == FLA_FLOAT || datatype == FLA_DOUBLE )
    r_val = TRUE;
  else
    r_val = FALSE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is_complex( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_complex( FLA_Obj A )
{
  FLA_Datatype datatype;
  FLA_Bool     r_val;

  datatype = FLA_Obj_datatype( A );

  if ( datatype == FLA_CONSTANT || datatype == FLA_COMPLEX || datatype == FLA_DOUBLE_COMPLEX )
    r_val = TRUE;
  else
    r_val = FALSE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is_single_precision( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_single_precision( FLA_Obj A )
{
  FLA_Datatype datatype;
  FLA_Bool     r_val;

  datatype = FLA_Obj_datatype( A );

  if ( datatype == FLA_CONSTANT || datatype == FLA_FLOAT || datatype == FLA_COMPLEX )
    r_val = TRUE;
  else
    r_val = FALSE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is_double_precision( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_double_precision( FLA_Obj A )
{
  FLA_Datatype datatype;
  FLA_Bool     r_val;

  datatype = FLA_Obj_datatype( A );

  if ( datatype == FLA_CONSTANT || datatype == FLA_DOUBLE || datatype == FLA_DOUBLE_COMPLEX )
    r_val = TRUE;
  else
    r_val = FALSE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is_scalar( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_scalar( FLA_Obj A )
{
  FLA_Bool r_val = FALSE;

  if ( FLA_Obj_length( A ) == 1 &&
       FLA_Obj_width( A ) == 1 )
    r_val = TRUE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is_vector( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_vector( FLA_Obj A )
{
  FLA_Bool r_val = FALSE;

  if ( FLA_Obj_length( A ) == 1 || FLA_Obj_width( A ) == 1 )
    r_val = TRUE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_has_zero_dim( )

 *************************************************************************** */

FLA_Bool FLA_Obj_has_zero_dim( FLA_Obj A )
{
  FLA_Bool r_val = FALSE;

  if ( FLA_Obj_length( A ) == 0 || FLA_Obj_width( A ) == 0 )
    r_val = TRUE;

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is_conformal_to( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is_conformal_to( FLA_Trans trans, FLA_Obj A, FLA_Obj B )
{
  FLA_Bool r_val = FALSE;

  if ( trans == FLA_NO_TRANSPOSE || trans == FLA_CONJ_NO_TRANSPOSE )
  {
    if ( FLA_Obj_length( A ) != FLA_Obj_length( B ) ||
         FLA_Obj_width( A )  != FLA_Obj_width( B ) )
      r_val = TRUE;
  }
  else
  {
    if ( FLA_Obj_width( A )  != FLA_Obj_length( B ) ||
         FLA_Obj_length( A ) != FLA_Obj_width( B ) )
      r_val = TRUE;
  }

  return r_val;
}


/* ***************************************************************************

   FLA_Obj_is( )

 *************************************************************************** */

FLA_Bool FLA_Obj_is( FLA_Obj A, FLA_Obj B )
{
  FLA_Bool r_val = FALSE;

  if ( A.base == B.base )
    r_val = TRUE;

  return r_val;
}

void FLA_F2C( fla_obj_is_f ) 
     ( F_INT *A, F_INT *B, F_INT *IERROR )
{
  *IERROR = FLA_Obj_is( *( ( FLA_Obj * ) A ),
                        *( ( FLA_Obj * ) B ) );
}


/* ***************************************************************************

   FLA_Obj_equals( )

 *************************************************************************** */

FLA_Bool FLA_Obj_equals( FLA_Obj A, FLA_Obj B )
{
  FLA_Datatype datatype_A;
  FLA_Datatype datatype_B;
  FLA_Datatype datatype;
  int          m, n, ldim_A, ldim_B;
  int          i, j;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Obj_equals_check( A, B );
#endif

  m      = FLA_Obj_length( A );
  n      = FLA_Obj_width( A );
  ldim_A = FLA_Obj_ldim( A );
  ldim_B = FLA_Obj_ldim( B );

  datatype_A = FLA_Obj_datatype( A );
  datatype_B = FLA_Obj_datatype( B );

  // If A is a non-FLA_CONSTANT object, then we should proceed based on the
  // value of datatype_A. In such a situation, either datatype_B is an exact
  // match and we're fine, or datatype_B is FLA_CONSTANT, in which case we're
  // also covered since FLA_CONSTANT encompassas all numerical types.
  // If A is an FLA_CONSTANT object, then we should proceed based on the value
  // of datatype_B. In this case, datatype_B is either a non-FLA_CONSTANT type,
  // which mirrors the second sub-case above, or datatype_B is FLA_CONSTANT,
  // in which case both types are FLA_CONSTANT and therefore we have to handle
  // that case. Only if both are FLA_CONSTANTs does the FLA_CONSTANT case
  // statement below execute.
  if ( datatype_A != FLA_CONSTANT )
    datatype = datatype_A;
  else
    datatype = datatype_B;

  switch ( datatype ){

  case FLA_CONSTANT:
  {
    // If both objects are FLA_CONSTANTs, then it doesn't matter which datatype
    // macro we use, as long as we use the same one for both A and B.
    int *buff_A = ( int * ) FLA_INT_PTR( A );
    int *buff_B = ( int * ) FLA_INT_PTR( B );

    for ( j = 0; j < n; j++ )
      for ( i = 0; i < m; i++ )
        if ( buff_A[ j * ldim_A + i ] != 
             buff_B[ j * ldim_B + i ] )
          return FALSE;

    break;
  }

  case FLA_INT:
  {
    int *buff_A = ( int * ) FLA_INT_PTR( A );
    int *buff_B = ( int * ) FLA_INT_PTR( B );

    for ( j = 0; j < n; j++ )
      for ( i = 0; i < m; i++ )
        if ( buff_A[ j * ldim_A + i ] != 
             buff_B[ j * ldim_B + i ] )
        {
          return FALSE;
        }

    break;
  }

  case FLA_FLOAT:
  {
    float *buff_A = ( float * ) FLA_FLOAT_PTR( A );
    float *buff_B = ( float * ) FLA_FLOAT_PTR( B );

    for ( j = 0; j < n; j++ )
      for ( i = 0; i < m; i++ )
        if ( buff_A[ j * ldim_A + i ] != 
             buff_B[ j * ldim_B + i ] )
        {
          return FALSE;
        }

    break;
  }

  case FLA_DOUBLE:
  {
    double *buff_A = ( double * ) FLA_DOUBLE_PTR( A );
    double *buff_B = ( double * ) FLA_DOUBLE_PTR( B );

    for ( j = 0; j < n; j++ )
      for ( i = 0; i < m; i++ )
        if ( buff_A[ j * ldim_A + i ] != 
             buff_B[ j * ldim_B + i ] )
        {
          return FALSE;
        }

    break;
  }

  case FLA_COMPLEX:
  {
    scomplex *buff_A = ( scomplex * ) FLA_COMPLEX_PTR( A );
    scomplex *buff_B = ( scomplex * ) FLA_COMPLEX_PTR( B );

    for ( j = 0; j < n; j++ )
      for ( i = 0; i < m; i++ )
        if ( buff_A[ j * ldim_A + i ].real != buff_B[ j * ldim_B + i ].real ||
             buff_A[ j * ldim_A + i ].imag != buff_B[ j * ldim_B + i ].imag )
        {
          return FALSE;
        }

    break;
  }

  case FLA_DOUBLE_COMPLEX:
  {
    dcomplex *buff_A = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( A );
    dcomplex *buff_B = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( B );

    for ( j = 0; j < n; j++ )
      for ( i = 0; i < m; i++ )
        if ( buff_A[ j * ldim_A + i ].real != buff_B[ j * ldim_B + i ].real ||
             buff_A[ j * ldim_A + i ].imag != buff_B[ j * ldim_B + i ].imag )
        {
          return FALSE;
        }

    break;
  }

  }

  return TRUE;
}

void FLA_F2C( fla_obj_equals_f ) 
     ( F_INT *A, F_INT *B, F_INT *IERROR )
{
  *IERROR = FLA_Obj_equals( *( ( FLA_Obj * ) A ),
                            *( ( FLA_Obj * ) B ) );
}


/* ***************************************************************************

   FLA_Obj_set_to_scalar( )

 *************************************************************************** */

FLA_Error FLA_Obj_set_to_scalar( FLA_Obj alpha, FLA_Obj A )
{
  FLA_Datatype datatype;
  int          m_A, n_A, ldim_A;
  int          i, j;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Obj_set_to_scalar_check( alpha, A );
#endif

  datatype = FLA_Obj_datatype( A );
  m_A      = FLA_Obj_length( A );
  n_A      = FLA_Obj_width( A );
  ldim_A   = FLA_Obj_ldim( A );

  switch ( datatype ){

  case FLA_FLOAT:
  {
    float *buff_A     = ( float * ) FLA_FLOAT_PTR( A );
    float *buff_alpha = ( float * ) FLA_FLOAT_PTR( alpha );
 
    for ( j = 0; j < n_A; j++ )
      for ( i = 0; i < m_A; i++ )
        buff_A[ j * ldim_A + i ] = *buff_alpha;

    break;
  }

  case FLA_DOUBLE:
  {
    double *buff_A     = ( double * ) FLA_DOUBLE_PTR( A );
    double *buff_alpha = ( double * ) FLA_DOUBLE_PTR( alpha );
 
    for ( j = 0; j < n_A; j++ )
      for ( i = 0; i < m_A; i++ )
        buff_A[ j * ldim_A + i ] = *buff_alpha;

    break;
  }

  case FLA_COMPLEX:
  {
    scomplex *buff_A     = ( scomplex * ) FLA_COMPLEX_PTR( A );
    scomplex *buff_alpha = ( scomplex * ) FLA_COMPLEX_PTR( alpha );
 
    for ( j = 0; j < n_A; j++ )
      for ( i = 0; i < m_A; i++ )
        buff_A[ j * ldim_A + i ] = *buff_alpha;

    break;
  }

  case FLA_DOUBLE_COMPLEX:
  {
    dcomplex *buff_A     = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( A );
    dcomplex *buff_alpha = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( alpha );
 
    for ( j = 0; j < n_A; j++ )
      for ( i = 0; i < m_A; i++ )
        buff_A[ j * ldim_A + i ] = *buff_alpha;

    break;
  }

  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_obj_set_to_scalar_f ) 
     ( F_INT *alpha, F_INT *A, F_INT *IERROR )
{
  *IERROR = FLA_Obj_set_to_scalar( *( ( FLA_Obj * ) alpha ),
                                   *( ( FLA_Obj * ) A     ) );
}


/* ***************************************************************************

   FLA_Obj_set_diagonal_to_scalar( )

 *************************************************************************** */

FLA_Error FLA_Obj_set_diagonal_to_scalar( FLA_Obj alpha, FLA_Obj A )
{
  FLA_Datatype datatype;
  int          i, min_m_n, ldim;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Obj_set_diagonal_to_scalar_check( alpha, A );
#endif

  datatype = FLA_Obj_datatype( A );
  ldim     = FLA_Obj_ldim( A );
  min_m_n  = FLA_Obj_min_dim( A );

  switch ( datatype ){

  case FLA_FLOAT:
  {
    float *buff_A     = ( float * ) FLA_FLOAT_PTR( A );
    float *buff_alpha = ( float * ) FLA_FLOAT_PTR( alpha );

    for ( i = 0; i < min_m_n; i++ )
      buff_A[ i*ldim + i ] = *buff_alpha;

    break;
  }

  case FLA_DOUBLE:
  {
    double *buff_A     = ( double * ) FLA_DOUBLE_PTR( A );
    double *buff_alpha = ( double * ) FLA_DOUBLE_PTR( alpha );

    for ( i = 0; i < min_m_n; i++ )
      buff_A[ i*ldim + i ] = *buff_alpha;

    break;
  }

  case FLA_COMPLEX:
  {
    scomplex *buff_A     = ( scomplex * ) FLA_COMPLEX_PTR( A );
    scomplex *buff_alpha = ( scomplex * ) FLA_COMPLEX_PTR( alpha );

    for ( i = 0; i < min_m_n; i++ )
    {
      buff_A[ i*ldim + i ].real = buff_alpha->real;
      buff_A[ i*ldim + i ].imag = buff_alpha->imag;
    }

    break;
  }

  case FLA_DOUBLE_COMPLEX:
  {
    dcomplex *buff_A     = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( A );
    dcomplex *buff_alpha = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( alpha );

    for ( i = 0; i < min_m_n; i++ )
    {
      buff_A[ i*ldim + i ].real = buff_alpha->real;
      buff_A[ i*ldim + i ].imag = buff_alpha->imag;
    }

    break;
  }

  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_obj_set_diagonal_to_scalar_f ) 
     ( F_INT *alpha, F_INT *A, F_INT *IERROR )
{
  *IERROR = FLA_Obj_set_diagonal_to_scalar( *( ( FLA_Obj * ) alpha ),
                                            *( ( FLA_Obj * ) A     ) );
}


/* ***************************************************************************

   FLA_Obj_set_to_identity( )

 *************************************************************************** */

FLA_Error FLA_Obj_set_to_identity( FLA_Obj A )
{
#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Obj_set_to_identity_check( A );
#endif

  FLA_Obj_set_to_scalar( FLA_ZERO, A );

  FLA_Obj_set_diagonal_to_scalar( FLA_ONE, A );

  return FLA_SUCCESS;
}

void FLA_F2C( fla_obj_set_to_identity_f )
     ( F_INT *A, F_INT *IERROR )
{
  *IERROR = FLA_Obj_set_to_identity( *( ( FLA_Obj * ) A ) );
}


/* ***************************************************************************

   FLA_Obj_add_to_diagonal( )

 *************************************************************************** */

FLA_Error FLA_Obj_add_to_diagonal( void* diag_value, FLA_Obj A )
{
  FLA_Datatype datatype;
  int          i, n, ldim;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Obj_add_to_diagonal_check( diag_value, A );
#endif

  datatype = FLA_Obj_datatype( A );
  ldim     = FLA_Obj_ldim( A );
  n        = FLA_Obj_min_dim( A );

  switch ( datatype ){

  case FLA_FLOAT:
  {
    float *buff_A    = ( float * ) FLA_FLOAT_PTR( A );
    float *value_ptr = ( float * ) diag_value;

    for ( i = 0; i < n; i++ )
      buff_A[ i*ldim + i ] += *value_ptr;

    break;
  }

  case FLA_DOUBLE:
  {
    double *buff_A    = ( double * ) FLA_DOUBLE_PTR( A );
    double *value_ptr = ( double * ) diag_value;

    for ( i = 0; i < n; i++ )
      buff_A[ i*ldim + i ] += *value_ptr;

    break;
  }

  case FLA_COMPLEX:
  {
    scomplex *buff_A    = ( scomplex * ) FLA_COMPLEX_PTR( A );
    scomplex *value_ptr = ( scomplex * ) diag_value;

    for ( i = 0; i < n; i++ )
    {
      buff_A[ i*ldim + i ].real += value_ptr->real;
      buff_A[ i*ldim + i ].imag += value_ptr->imag;
    }

    break;
  }

  case FLA_DOUBLE_COMPLEX:
  {
    dcomplex *buff_A    = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( A );
    dcomplex *value_ptr = ( dcomplex * ) diag_value;

    for ( i = 0; i < n; i++ )
    {
      buff_A[ i*ldim + i ].real += value_ptr->real;
      buff_A[ i*ldim + i ].imag += value_ptr->imag;
    }

    break;
  }

  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_obj_add_to_diagonal_f ) 
     ( F_INT *value, F_INT *A, F_INT *IERROR )
{
  *IERROR = FLA_Obj_add_to_diagonal(  ( ( void    * ) value ),
                                     *( ( FLA_Obj * ) A     ) );
}


/* ***************************************************************************

   FLA_Obj_shift_diagonal( )

 *************************************************************************** */

FLA_Error FLA_Obj_shift_diagonal( FLA_Obj sigma, FLA_Obj A )
{
  FLA_Datatype datatype_A;
  FLA_Datatype datatype_sigma;
  int          i, min_m_n, ldim_A;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Obj_shift_diagonal_check( sigma, A );
#endif

  ldim_A         = FLA_Obj_ldim( A );
  min_m_n        = FLA_Obj_min_dim( A );
  datatype_A     = FLA_Obj_datatype( A );
  datatype_sigma = FLA_Obj_datatype( sigma );
  
  switch( datatype_A ){

  case FLA_FLOAT:
  {
    float *buff_A     = ( float * ) FLA_FLOAT_PTR( A );
    float *buff_sigma = ( float * ) FLA_FLOAT_PTR( sigma );

    for ( i = 0; i < min_m_n; i++ )
      *( buff_A + i*ldim_A + i ) += *buff_sigma;

    break;
  }

  case FLA_DOUBLE:
  {
    double *buff_A     = ( double * ) FLA_DOUBLE_PTR( A );
    double *buff_sigma = ( double * ) FLA_DOUBLE_PTR( sigma );

    for ( i = 0; i < min_m_n; i++ )
      *( buff_A + i*ldim_A + i ) += *buff_sigma;

    break;
  }

  case FLA_COMPLEX:
  {
    if ( datatype_sigma == FLA_COMPLEX )
    {
      scomplex *buff_A     = ( scomplex * ) FLA_COMPLEX_PTR( A );
      scomplex *buff_sigma = ( scomplex * ) FLA_COMPLEX_PTR( sigma );
  
      for ( i = 0; i < min_m_n; i++ )
      {
        ( buff_A + i*ldim_A + i )->real += buff_sigma->real;
        ( buff_A + i*ldim_A + i )->imag += buff_sigma->imag;
      }
    }
    else
    {
      scomplex *buff_A     = ( scomplex * ) FLA_COMPLEX_PTR( A );
      float    *buff_sigma = ( float    * ) FLA_FLOAT_PTR( sigma );
  
      for ( i = 0; i < min_m_n; i++ )
      {
        ( buff_A + i*ldim_A + i )->real += *buff_sigma;
      }
    }

    break;
  }

  case FLA_DOUBLE_COMPLEX:
  {
    if ( datatype_sigma == FLA_DOUBLE_COMPLEX )
    {
      dcomplex *buff_A     = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( A );
      dcomplex *buff_sigma = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( sigma );

      for ( i = 0; i < min_m_n; i++ )
      {
        ( buff_A + i*ldim_A + i )->real += buff_sigma->real;
        ( buff_A + i*ldim_A + i )->imag += buff_sigma->imag;
      }
    }
    else
    {
      dcomplex *buff_A     = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( A );
      double   *buff_sigma = ( double   * ) FLA_DOUBLE_PTR( sigma );

      for ( i = 0; i < min_m_n; i++ )
      {
        ( buff_A + i*ldim_A + i )->real += *buff_sigma;
      }
    }

    break;
  }

  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_obj_shift_diagonal_f )
     ( F_INT *sigma, F_INT *A, F_INT *IERROR )
{
  *IERROR = FLA_Obj_shift_diagonal( *( ( FLA_Obj * ) sigma ), 
                                    *( ( FLA_Obj * ) A     ) );
}


/* ***************************************************************************

   FLA_Obj_scale_diagonal( )

 *************************************************************************** */

FLA_Error FLA_Obj_scale_diagonal( FLA_Obj alpha, FLA_Obj A )
{
  FLA_Datatype datatype_A;
  FLA_Datatype datatype_alpha;
  int          i, min_m_n, ldim_A;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Obj_scale_diagonal_check( alpha, A );
#endif

  ldim_A         = FLA_Obj_ldim( A );
  min_m_n        = FLA_Obj_min_dim( A );
  datatype_A     = FLA_Obj_datatype( A );
  datatype_alpha = FLA_Obj_datatype( alpha );
  
  switch( datatype_A ){

  case FLA_FLOAT:
  {
    float *buff_A     = ( float * ) FLA_FLOAT_PTR( A );
    float *buff_alpha = ( float * ) FLA_FLOAT_PTR( alpha );

    for ( i = 0; i < min_m_n; i++ )
      *( buff_A + i*ldim_A + i ) *= *buff_alpha;

    break;
  }

  case FLA_DOUBLE:
  {
    double *buff_A     = ( double * ) FLA_DOUBLE_PTR( A );
    double *buff_alpha = ( double * ) FLA_DOUBLE_PTR( alpha );

    for ( i = 0; i < min_m_n; i++ )
      *( buff_A + i*ldim_A + i ) *= *buff_alpha;

    break;
  }

  case FLA_COMPLEX:
  {
    if ( datatype_alpha == FLA_COMPLEX )
    {
      scomplex *buff_A     = ( scomplex * ) FLA_COMPLEX_PTR( A );
      scomplex *buff_alpha = ( scomplex * ) FLA_COMPLEX_PTR( alpha );
  
      for ( i = 0; i < min_m_n; i++ )
      {
        ( buff_A + i*ldim_A + i )->real *= buff_alpha->real;
        ( buff_A + i*ldim_A + i )->imag *= buff_alpha->imag;
      }
    }
    else
    {
      scomplex *buff_A     = ( scomplex * ) FLA_COMPLEX_PTR( A );
      float    *buff_alpha = ( float    * ) FLA_FLOAT_PTR( alpha );
  
      for ( i = 0; i < min_m_n; i++ )
      {
        ( buff_A + i*ldim_A + i )->real *= *buff_alpha;
      }
    }

    break;
  }

  case FLA_DOUBLE_COMPLEX:
  {
    if ( datatype_alpha == FLA_DOUBLE_COMPLEX )
    {
      dcomplex *buff_A     = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( A );
      dcomplex *buff_alpha = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( alpha );

      for ( i = 0; i < min_m_n; i++ )
      {
        ( buff_A + i*ldim_A + i )->real *= buff_alpha->real;
        ( buff_A + i*ldim_A + i )->imag *= buff_alpha->imag;
      }
    }
    else
    {
      dcomplex *buff_A     = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( A );
      double   *buff_alpha = ( double   * ) FLA_DOUBLE_PTR( alpha );

      for ( i = 0; i < min_m_n; i++ )
      {
        ( buff_A + i*ldim_A + i )->real *= *buff_alpha;
      }
    }

    break;
  }

  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_obj_scale_diagonal_f )
     ( F_INT *alpha, F_INT *A, F_INT *IERROR )
{
  *IERROR = FLA_Obj_scale_diagonal( *( ( FLA_Obj * ) alpha ), 
                                    *( ( FLA_Obj * ) A     ) );
}


/* *************************************************************************

   FLA_Obj_show( )

 *************************************************************************** */

#define BUFFER(i,j,ldim) buffer[(j)*(ldim)+(i)]

FLA_Error FLA_Obj_show( char *s1, FLA_Obj A, char *format, char *s2 )
{
  FLA_Datatype datatype;
  int          m, n, i, j, ldim;

  datatype = FLA_Obj_datatype( A );
  m        = FLA_Obj_length  ( A );
  n        = FLA_Obj_width   ( A );
  ldim     = FLA_Obj_ldim    ( A );

  printf( "%s\n", s1 );

  switch ( datatype ){

  case FLA_FLOAT:
  {
    float *buffer = ( float * ) FLA_FLOAT_PTR( A );

    for ( i = 0; i < m; i++ )
    {
      for ( j = 0; j < n; j++ )
      {
        printf( format, BUFFER( i, j, ldim ) );
        printf( " " );
      }
      printf( "\n" );
    }
    printf("%s\n", s2 );
    break;
  }

  case FLA_DOUBLE:
  {
    double *buffer = ( double * ) FLA_DOUBLE_PTR( A );

    for ( i = 0; i < m; i++ )
    {
      for ( j = 0; j < n; j++ )
      {
        printf( format, BUFFER( i, j, ldim ) );
        printf( " " );
      }
      printf( "\n" );
    }
    printf("%s\n", s2 );
    break;
  }

  case FLA_COMPLEX:
  {
    scomplex *buffer = ( scomplex * ) FLA_COMPLEX_PTR( A );

    for ( i = 0; i < m; i++ )
    {
      for ( j = 0; j < n; j++ )
      {
        printf( format, BUFFER(i,j,ldim).real, BUFFER(i,j,ldim).imag );
        printf( " " );
      }
      printf( "\n" );
    }
    printf("%s\n", s2 );
    break;
  }

  case FLA_DOUBLE_COMPLEX:
  {
    dcomplex *buffer = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( A );

    for ( i = 0; i < m; i++ )
    {
      for ( j = 0; j < n; j++ )
      {
        printf( format, BUFFER(i,j,ldim).real, BUFFER(i,j,ldim).imag );
        printf( " " );
      }
      printf( "\n" );
    }
    printf("%s\n", s2 );
    break;
  }

  case FLA_INT:
  {
    int *buffer = ( int * ) FLA_INT_PTR( A );

    for ( i = 0; i < m; i++ )
    {
      for ( j = 0; j < n; j++ )
      {
        printf( format, BUFFER( i, j, ldim ) );
        printf( " " );
      }
      printf( "\n" );
    }
    printf("%s\n", s2 );
    break;
  }

  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_obj_show_f )
     ( F_INT *s1, F_INT *A, F_INT *format, F_INT *s2, F_INT *IERROR )
{
  *IERROR = FLA_Obj_show(  ( ( char    * ) s1     ), 
                          *( ( FLA_Obj * ) A      ), 
                           ( ( char    * ) format ), 
                           ( ( char    * ) s2     ) );
}


/* ***************************************************************************

   FLA_Copy_submatrix_to_global()

 *************************************************************************** */

FLA_Error FLA_Copy_submatrix_to_global( FLA_Trans trans, int m, int n, void* A_buffer, int ldim, int i, int j, FLA_Obj B )
{
  FLA_Obj  A;
  FLA_Obj  BTL, BTR, 
           BBL, Bij;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Copy_submatrix_to_global_check( trans, m, n, A_buffer, ldim, i, j, B );
#endif

  FLA_Part_2x2( B,  &BTL, &BTR,
                    &BBL, &Bij,     i, j, FLA_TL );

  FLA_Obj_create_without_buffer( FLA_Obj_datatype( B ), m, n, &A );
  FLA_Obj_attach_buffer( A_buffer, ldim, &A );

  FLA_Copy_t( trans, A, Bij );

  FLA_Obj_free_without_buffer( &A );

  return FLA_SUCCESS;
}


/* ***************************************************************************

   FLA_Copy_global_to_submatrix()

 *************************************************************************** */

FLA_Error FLA_Copy_global_to_submatrix( FLA_Trans trans, int i, int j, FLA_Obj A, int m, int n, void* B_buffer, int ldim )
{
  FLA_Obj  B;
  FLA_Obj  ATL, ATR, 
           ABL, Aij;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Copy_global_to_submatrix_check( trans, i, j, A, m, n, B_buffer, ldim );
#endif

  FLA_Part_2x2( A,  &ATL, &ATR,
                    &ABL, &Aij,     i, j, FLA_TL );

  FLA_Obj_create_without_buffer( FLA_Obj_datatype( A ), m, n, &B );
  FLA_Obj_attach_buffer( B_buffer, ldim, &B );

  FLA_Copy_t( trans, Aij, B );

  FLA_Obj_free_without_buffer( &B );

  return FLA_SUCCESS;
}


/* ***************************************************************************

   FLA_Axpy_submatrix_to_global()

 *************************************************************************** */

FLA_Error FLA_Axpy_submatrix_to_global( FLA_Trans trans, FLA_Obj alpha, int m, int n, void* X_buffer, int ldim, int i, int j, FLA_Obj Y )
{
  FLA_Obj  X;
  FLA_Obj  YTL, YTR, 
           YBL, Yij;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Axpy_submatrix_to_global_check( trans, alpha, m, n, X_buffer, ldim, i, j, Y );
#endif

  FLA_Part_2x2( Y,  &YTL, &YTR,
                    &YBL, &Yij,     i, j, FLA_TL );

  FLA_Obj_create_without_buffer( FLA_Obj_datatype( Y ), m, n, &X );
  FLA_Obj_attach_buffer( X_buffer, ldim, &X );

  FLA_Axpy_t( trans, alpha, X, Yij );

  FLA_Obj_free_without_buffer( &X );

  return FLA_SUCCESS;
}


/* ***************************************************************************

   FLA_Axpy_global_to_submatrix()

 *************************************************************************** */

FLA_Error FLA_Axpy_global_to_submatrix( FLA_Trans trans, FLA_Obj alpha, int i, int j, FLA_Obj X, int m, int n, void* Y_buffer, int ldim )
{
  FLA_Obj  Y;
  FLA_Obj  XTL, XTR, 
           XBL, Xij;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Axpy_global_to_submatrix_check( trans, alpha, i, j, X, m, n, Y_buffer, ldim );
#endif

  FLA_Part_2x2( X,  &XTL, &XTR,
                    &XBL, &Xij,     i, j, FLA_TL );

  FLA_Obj_create_without_buffer( FLA_Obj_datatype( X ), m, n, &Y );
  FLA_Obj_attach_buffer( Y_buffer, ldim, &Y );

  FLA_Axpy_t( trans, alpha, Xij, Y );

  FLA_Obj_free_without_buffer( &Y );

  return FLA_SUCCESS;
}

