/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h" 


int main( int argc, char *argv[] ) 
{ 
   int
      verbose       = FLASH_VERBOSE,
      sorting       = FLASH_SORTING,
      caching       = FLASH_CACHING,
      data_affinity = FLASH_DATA_AFFINITY,
      size      = FLASH_SIZE,
      nb_alg    = FLASH_NB_ALG,
      n_threads = FLASH_N_THREADS;

   FLA_Datatype
      datatype = FLA_DOUBLE;
   
   FLA_Obj
      A, AH;
   
   FLA_Init(); 

   FLASH_Queue_set_num_threads( n_threads );
   FLASH_Queue_set_verbose_output( verbose );
   FLASH_Queue_set_sorting( sorting );
   FLASH_Queue_set_caching( caching );
   FLASH_Queue_set_data_affinity( data_affinity );   

   FLA_Obj_create( datatype, size, size, &A ); 
   FLA_Random_spd_matrix( FLA_LOWER_TRIANGULAR, A );

   FLASH_Obj_create_hier_copy_of_flat( A, 1, &nb_alg, &AH );  

   printf( "%d[%d,%d]\n", FLASH_OBJ_PTR_ID( AH ),
           FLA_Obj_length( AH ), FLA_Obj_width( AH ) );
   printf( "%d\n%d\n", nb_alg, n_threads );
         
   FLASH_Chol( FLA_LOWER_TRIANGULAR, AH );
      
   FLASH_Obj_flatten( AH, A );
   FLASH_Obj_free( &AH );
   
   FLA_Obj_free( &A ); 
   
   FLA_Finalize(); 
   
   return 0; 
}
