/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Invert( FLA_Obj alpha )
{
  FLA_Datatype datatype;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Invert_check( alpha );
#endif

  datatype = FLA_Obj_datatype( alpha );
  
  switch ( datatype ){

  case FLA_FLOAT:
  {
    float *buff_alpha = ( float * ) FLA_FLOAT_PTR( alpha );

    *buff_alpha = 1.0F / *buff_alpha;

    break;
  }

  case FLA_DOUBLE:
  {
    double *buff_alpha = ( double * ) FLA_DOUBLE_PTR( alpha );

    *buff_alpha = 1.0 / *buff_alpha;

    break;
  }

  case FLA_COMPLEX:
  {
    scomplex *buff_alpha = ( scomplex * ) FLA_COMPLEX_PTR( alpha );
    float     temp;

    temp = 1.0F / ( buff_alpha->real * buff_alpha->real + 
                    buff_alpha->imag * buff_alpha->imag );

    buff_alpha->real = buff_alpha->real *  temp; 
    buff_alpha->imag = buff_alpha->imag * -temp; 

    break;
  }

  case FLA_DOUBLE_COMPLEX:
  {
    dcomplex *buff_alpha = ( dcomplex * ) FLA_DOUBLE_COMPLEX_PTR( alpha );
    double    temp;

    temp = 1.0 / ( buff_alpha->real * buff_alpha->real + 
                   buff_alpha->imag * buff_alpha->imag );

    buff_alpha->real = buff_alpha->real *  temp; 
    buff_alpha->imag = buff_alpha->imag * -temp; 

    break;
  }

  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_invert_f )
     ( F_INT *alpha, F_INT *IERROR )
{
  *IERROR = FLA_Invert( *( ( FLA_Obj * ) alpha ) );
}

