/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


PLA_Error PLA_Local_chol_check( PLA_Uplo uplo, 
				PLA_Obj  A )
/*----------------------------------------------------------------------------

   PLA_Local_chol_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_LU_nopiv_check( PLA_Obj A )
/*----------------------------------------------------------------------------

   PLA_Local_LU_nopiv_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
   }

   return PLA_SUCCESS;
}


PLA_Error PLA_Local_LU_piv_check( PLA_Obj A,
				  PLA_Obj p )
/*----------------------------------------------------------------------------

   PLA_Local_LU_piv_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( A, p, "PLA_Local_LU_piv: objects not on same processor" );
   }

   return PLA_SUCCESS;
}


PLA_Error PLA_Local_QR_check( PLA_Obj A, 
			      PLA_Obj t,
			      PLA_Obj S )
/*----------------------------------------------------------------------------

   PLA_Local_QR_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( A, S, "PLA_Local_QR: objects not on same processor" );
      PLA_Check_consistent_owners( t, S, "PLA_Local_QR: objects not on same processor" );
   }

   return PLA_SUCCESS;
}


PLA_Error PLA_Local_LQ_check( PLA_Obj A,
			      PLA_Obj t, 
			      PLA_Obj S )
/*----------------------------------------------------------------------------

   PLA_Local_LQ_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( A, S, "PLA_Local_LQ: objects not on same processor" );
      PLA_Check_consistent_owners( t, S, "PLA_Local_LQ: objects not on same processor" );
   }

   return PLA_SUCCESS;
}


PLA_Error PLA_Local_HRD_check( PLA_Obj A,
			       PLA_Obj t, 
			       int     ilo, 
			       int     ihi )
/*----------------------------------------------------------------------------

   PLA_Local_HRD_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( A, t, "PLA_Local_HRD: objects not on same processor" );
   }

   return PLA_SUCCESS;
}
