/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#ifndef PLA_OBJ_H
#define PLA_OBJ_H


typedef struct PLA_Obj_struct 
{
      void *buffer;
      
      PLA_Datatype
         datatype;
      int
         m,
	 n,
	 ldim;
      
      /* Parameters for OOC matrices */
      int
         offset,
	 fp;

      /* Parameter for matrices and vectors distributed among processors */
      PLA_Objtype
         objtype;          /* Type of distribution among processors      */
      int  
         alignM,           /* Alignment to template                      */ 
	 alignN,           /* Alignment to template                      */
	 ownerRow,         /* Index of processor row that owns object    */
	 ownerCol;         /* Index of processor column that owns object */
      
      PLA_Temp templ;      /* Template used for distribution             */

} PLA_Base_obj;


typedef struct PLA_Obj_view 
{
      int 
         offm, offn,
	 m, n;
      
      PLA_Base_obj *base;

} PLA_Obj;


/*----------------------------------------------------------------------------

               <----------n---------->   
	                  |---ownerCol---|
       -----------------------------------
       |          ^       |	         |			
       |          |       |              |
       |        alignM    |              |
       |          |       |              |
       |          v       |              |
   ^   |       ***********************   |
   |   |   a   *          |          *   |
   |   |   l   *          |          *   |
   |   |<--i-->*          |          *   |
   |   |   g   *          |          *   |
   |   |   n   *          |          *   |
   |   |   N   *          |          *   |
   |   |       *          |          *   |
   m - |-------*----------|----------*---|
   | | |       *          |          *   |
   | o |       *          |          *   |
   | w |       *          |          *   |
   | n |       *          |          *   |
   | e |       *          |          *   |
   | r |       *          |          *   |
   | R |       *          |          *   |
   v o |       ***********************   |
     w |                  |              |
     | |                  |              |
     - -----------------------------------

----------------------------------------------------------------------------*/

#endif
