/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#ifndef PLA_TEMPLATE_H
#define PLA_TEMPLATE_H


struct PLA_Template_struct
{
      MPI_Comm
         commAll,
	 commRow,
	 commCol;
      
      int
         mb,       /* Block size for the inducing column vector */
	 colDistr,
	 nb,       /* Block size for the inducing row vector    */
	 rowDistr,
	 
	 me,
	 myrow,
	 mycol,
	 
	 nprocs,
	 nprows,
	 npcols;
}; 

typedef struct PLA_Template_struct  PLA_Template;

typedef struct PLA_Template_struct *  PLA_Temp;


/*----------------------------------------------------------------------------

             |-npcols----------------|
                             |-mycol-|
                     <---nb-->
   -      ^  -------------------------
   |      |  |   c   |       |       |
   |      |  |   o   |       |       |
   n      mb |   l   |   rowDistr    |
   p      |  |   D   |       |       |
   r   -  v  |---i---|-------|-------|
   o   |     |   s   |       |       |
   w   |     |   t   |       |       |
   s myrow   |   r   |       |       |
   |   |     |       |       |       |
   -   -     -------------------------

----------------------------------------------------------------------------*/

#endif
