/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#ifndef PLA_LOCAL_BLAS_PROTOTYPES_H
#define PLA_LOCAL_BLAS_PROTOTYPES_H


/**************** PLA_Local_blas1.c ****************/

PLA_Error PLA_Local_asum( PLA_Obj x, PLA_Obj asum_x );
PLA_Error PLA_Local_axpy( PLA_Obj alpha, PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_copy( PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_dot( PLA_Obj x, PLA_Obj y, PLA_Obj rho );
PLA_Error PLA_Local_dotc( PLA_Obj x, PLA_Obj y, PLA_Obj rho );
PLA_Error PLA_Local_iamax( PLA_Obj x, PLA_Obj index );
PLA_Error PLA_Local_inv_scal( PLA_Obj alpha, PLA_Obj A );
PLA_Error PLA_Local_nrm2( PLA_Obj x, PLA_Obj nrm_x );
PLA_Error PLA_Local_scal( PLA_Obj alpha, PLA_Obj A );
PLA_Error PLA_Local_swap( PLA_Obj A, PLA_Obj B );

PLA_Error PLA_Local_axpy_x( PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_axpy_y( PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj beta, PLA_Obj B );
PLA_Error PLA_Local_copy_x( PLA_Trans trans, PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_dot_x( PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj beta, PLA_Obj rho );
PLA_Error PLA_Local_scal_x( PLA_Trans uplo, PLA_Obj alpha, PLA_Obj A );


/**************** PLA_Local_blas2.c ****************/

PLA_Error PLA_Local_gemv( PLA_Trans transa, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );
PLA_Error PLA_Local_ger( PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj A );
PLA_Error PLA_Local_gerc( PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj A );
PLA_Error PLA_Local_hemv( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );
PLA_Error PLA_Local_her( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj x, PLA_Obj A );
PLA_Error PLA_Local_her2( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj A );
PLA_Error PLA_Local_symv( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );
PLA_Error PLA_Local_syr( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj x, PLA_Obj A );
PLA_Error PLA_Local_syr2( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj A );
PLA_Error PLA_Local_trmv( PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj A, PLA_Obj x );
PLA_Error PLA_Local_trsv( PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj A, PLA_Obj x );

PLA_Error PLA_Local_trmv_sx( PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );
PLA_Error PLA_Local_trsv_sx( PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );


/**************** PLA_Local_blas3.c ****************/

PLA_Error PLA_Local_gemm( PLA_Trans transa, PLA_Trans transb, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_hemm( PLA_Side side, PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_herk( PLA_Uplo uplo, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_her2k( PLA_Uplo uplo, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_symm( PLA_Side side, PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_syrk( PLA_Uplo uplo, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_syr2k( PLA_Uplo uplo, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_trmm( PLA_Side side, PLA_Uplo uplo, PLA_Trans trans, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_trsm( PLA_Side side, PLA_Uplo uplo, PLA_Trans trans, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj B );

PLA_Error PLA_Local_trmm_sx( PLA_Side side, PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_trsm_sx( PLA_Side side, PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );


/**************** PLA_Check_local_blas1.c ****************/

PLA_Error PLA_Local_asum_check( PLA_Obj x, PLA_Obj asum_x );
PLA_Error PLA_Local_axpy_check( PLA_Obj alpha, PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_copy_check( PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_dot_check( PLA_Obj x, PLA_Obj y, PLA_Obj rho );
PLA_Error PLA_Local_dotc_check( PLA_Obj x, PLA_Obj y, PLA_Obj rho );
PLA_Error PLA_Local_iamax_check( PLA_Obj x, PLA_Obj index );
PLA_Error PLA_Local_inv_scal_check( PLA_Obj alpha, PLA_Obj A );
PLA_Error PLA_Local_nrm2_check( PLA_Obj x, PLA_Obj nrm_x );
PLA_Error PLA_Local_scal_check( PLA_Obj alpha, PLA_Obj A );
PLA_Error PLA_Local_swap_check( PLA_Obj A, PLA_Obj B );

PLA_Error PLA_Local_axpy_x_check( PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_axpy_y_check( PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj beta, PLA_Obj B );
PLA_Error PLA_Local_copy_x_check( PLA_Trans trans, PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_dot_x_check( PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj beta, PLA_Obj rho );
PLA_Error PLA_Local_scal_x_check( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj A );


/**************** PLA_Check_local_blas2.c ****************/

PLA_Error PLA_Local_gemv_check( PLA_Trans transa, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );
PLA_Error PLA_Local_ger_check( PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj A );
PLA_Error PLA_Local_gerc_check( PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj A );
PLA_Error PLA_Local_hemv_check( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );
PLA_Error PLA_Local_her_check( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj x, PLA_Obj A );
PLA_Error PLA_Local_her2_check( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj A );
PLA_Error PLA_Local_symv_check( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );
PLA_Error PLA_Local_syr_check( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj x, PLA_Obj A );
PLA_Error PLA_Local_syr2_check( PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj x, PLA_Obj y, PLA_Obj A );
PLA_Error PLA_Local_trmv_check( PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj A, PLA_Obj x );
PLA_Error PLA_Local_trsv_check( PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj A, PLA_Obj x );

PLA_Error PLA_Local_trmv_sx_check( PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );
PLA_Error PLA_Local_trsv_sx_check( PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj x, PLA_Obj beta, PLA_Obj y );


/**************** PLA_Check_local_blas3.c ****************/

PLA_Error PLA_Local_gemm_check( PLA_Trans transa, PLA_Trans transb, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_hemm_check( PLA_Side side, PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_herk_check( PLA_Uplo uplo, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_her2k_check( PLA_Uplo uplo, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_symm_check( PLA_Side side, PLA_Uplo uplo, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_syrk_check( PLA_Uplo uplo, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_syr2k_check( PLA_Uplo uplo, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_trmm_check( PLA_Side side, PLA_Uplo uplo, PLA_Trans trans, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Local_trsm_check( PLA_Side side, PLA_Uplo uplo, PLA_Trans trans, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj B );

PLA_Error PLA_Local_trmm_sx_check( PLA_Side side, PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );
PLA_Error PLA_Local_trsm_sx_check( PLA_Side side, PLA_Uplo uplo, PLA_Trans transa, PLA_Diag diag, PLA_Obj alpha, PLA_Obj A, PLA_Obj B, PLA_Obj beta, PLA_Obj C );


#endif
