/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#ifndef PLA_MACRO_PTR_DEFS_H
#define PLA_MACRO_PTR_DEFS_H


#include "PLA_FLA_external.h"


#define PLA_CONSTANT_S_OFFSET  0
#define PLA_CONSTANT_D_OFFSET  ( PLA_Type_size(PLA_DOUBLE) )
#define PLA_CONSTANT_C_OFFSET  ( PLA_Type_size(PLA_DOUBLE) * 2 )
#define PLA_CONSTANT_Z_OFFSET  ( PLA_Type_size(PLA_DOUBLE) * 2 + PLA_Type_size(PLA_COMPLEX) )
#define PLA_CONSTANT_I_OFFSET  ( PLA_Type_size(PLA_DOUBLE) * 2 + PLA_Type_size(PLA_COMPLEX) + PLA_Type_size(PLA_DOUBLE_COMPLEX) )


#define PLA_FLOAT_PTR( x ) \
   ( ((x).base)->datatype == PLA_CONSTANT ? \
     (float *)    ( ( (char *)     ((x).base)->buffer ) + PLA_CONSTANT_S_OFFSET ) : \
                  ( ( (float *)    ((x).base)->buffer ) + PLA_Obj_local_offn(x) * ((x).base)->ldim + PLA_Obj_local_offm(x) ) \
     )
#define PLA_DOUBLE_PTR( x ) \
   ( ((x).base)->datatype == PLA_CONSTANT ? \
     (double *)   ( ( (char *)     ((x).base)->buffer ) + PLA_CONSTANT_D_OFFSET ) : \
                  ( ( (double *)   ((x).base)->buffer ) + PLA_Obj_local_offn(x) * ((x).base)->ldim + PLA_Obj_local_offm(x) ) \
     )
#define PLA_COMPLEX_PTR( x ) \
   ( ((x).base)->datatype == PLA_CONSTANT ? \
     (complex *)  ( ( (char *)     ((x).base)->buffer ) + PLA_CONSTANT_C_OFFSET ) : \
                  ( ( (complex *)  ((x).base)->buffer ) + PLA_Obj_local_offn(x) * ((x).base)->ldim + PLA_Obj_local_offm(x) ) \
     )
#define PLA_DOUBLE_COMPLEX_PTR( x ) \
   ( ((x).base)->datatype == PLA_CONSTANT ? \
     (dcomplex *) ( ( (char *)     ((x).base)->buffer ) + PLA_CONSTANT_Z_OFFSET ) : \
                  ( ( (dcomplex *) ((x).base)->buffer ) + PLA_Obj_local_offn(x) * ((x).base)->ldim + PLA_Obj_local_offm(x) ) \
     )
#define PLA_INT_PTR( x ) \
   ( ((x).base)->datatype == PLA_CONSTANT ? \
     (int *)      ( ( (char *)     ((x).base)->buffer ) + PLA_CONSTANT_I_OFFSET ) : \
                  ( ( (int *)      ((x).base)->buffer ) + PLA_Obj_local_offn(x) * ((x).base)->ldim + PLA_Obj_local_offm(x) ) \
     )


#endif
