/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#ifndef PLA_UTIL_PROTOTYPES_H
#define PLA_UTIL_PROTOTYPES_H


/**************** PLA_Copy.c ****************/

PLA_Error PLA_Copy( PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Copy_x( PLA_Trans trans, PLA_Obj A, PLA_Obj B );


/**************** PLA_Reduce.c ****************/

PLA_Error PLA_Reduce( PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Reduce_x( PLA_Op op, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj B );


/**************** PLA_Util.c ****************/

MPI_Op PLA_MPI_Op( PLA_Op op );
MPI_Datatype PLA_MPI_Datatype( PLA_Datatype datatype );
PLA_Error PLA_Scalar_copy( PLA_Obj alpha, PLA_Obj *alpha_cpy );


/**************** Misc ****************/

PLA_Error PLA_Random_matrix( PLA_Obj A );
PLA_Error PLA_Sqrt( PLA_Obj alpha );
PLA_Error PLA_Invert( PLA_Obj alpha );


/**************** PLA_Check_util.c ****************/

PLA_Error PLA_Copy_x_check( PLA_Trans trans, PLA_Obj A, PLA_Obj B );
PLA_Error PLA_Reduce_x_check( PLA_Op op, PLA_Trans trans, PLA_Obj alpha, PLA_Obj A, PLA_Obj B );

PLA_Error PLA_Sqrt_check( PLA_Obj alpha );
PLA_Error PLA_Invert_check( PLA_Obj alpha );


#endif
