/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


int PLA_printf( char *string, ... )
/*----------------------------------------------------------------------------

Purpose :  printf for distributed memory architectures

IN    string     input for printf

Return :    number of characters printed out on processor

----------------------------------------------------------------------------*/
{
   int characters = 0;
   
   // Only printf on processor zero
   if ( PLA_Temp_comm_all_rank( PLA_TEMP_WORLD ) == 0 )
   {
      va_list valist;
      
      // Initialize the va_list
      va_start( valist, string ); 
      
      // Call vprintf
      characters = vprintf( string, valist ); 
      
      // Cleanup the va_list
      va_end( valist ); 
   }
   
   return characters;
}


void PLA_Abort( char *s1,
                int  line,
                char *file )
/*----------------------------------------------------------------------------

Purpose :  Report an error

IN    s1         error message
IN    line       line number of error
IN    file       file of error

----------------------------------------------------------------------------*/
{
   PLA_printf( "%s:  line %d in file %s\nAborting\n\n", s1, line, file );
   
   exit(0);
}
