/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


int PLA_memcounter = 0;


void * PLA_malloc( int size )
/*----------------------------------------------------------------------------

Purpose :  Allocate memory

IN    size    length of memory to allocate

----------------------------------------------------------------------------*/
{
   void
      *buffer;
   
   if ( size == 0 )
      buffer = NULL;
   else
   {
      buffer = malloc( size );
      
      if ( buffer == NULL )
	 PLA_Abort( "malloc failed", __LINE__, __FILE__ );
      
#ifdef FLA_ENABLE_MEM_COUNTER
      PLA_memcounter += 1;
#endif
   }
  
  return buffer;
}


void PLA_free( void *buffer )
/*----------------------------------------------------------------------------

Purpose :  Free memory

IN    buffer    memory to free

----------------------------------------------------------------------------*/
{
   if ( buffer != NULL )
   {      
      free( buffer );
      
#ifdef FLA_ENABLE_MEM_COUNTER
      PLA_memcounter -= 1;
#endif
   }
   
   return;
}
