/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


PLA_Error create_problem( PLA_Obj, PLA_Obj, PLA_Obj );


PLA_Error create_problem( PLA_Obj A, PLA_Obj x, PLA_Obj b )
{
   PLA_Obj
      A_cpy;

   // assume always use PLA_Datatype:  PLA_DOUBLE   
   double d_m = (double) PLA_Obj_length( A );
   
   PLA_Random_matrix( A );
   PLA_Random_matrix( x );
   PLA_Random_matrix( b );
   
   PLA_Obj_add_to_diagonal( &d_m, A );
   
   // assume x and b are multiscalars
   PLA_Mscalar_create_conf_to( PLA_NO_TRANSPOSE, A,
			       PLA_INHERIT, PLA_INHERIT,
			       &A_cpy );
   PLA_Copy( A, A_cpy );
   
   
   PLA_Local_symv( PLA_LOWER_TRIANGULAR, PLA_ONE, 
		   A_cpy, x, PLA_ZERO, b );
   
   
   PLA_Copy( A_cpy, A ); 
   PLA_Obj_free( &A_cpy );

   return PLA_SUCCESS;
}
