/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


PLA_Error PLA_Invert( PLA_Obj alpha )
/*----------------------------------------------------------------------------

Purpose :  Invert object

OUT     alpha      object must be a scalar

----------------------------------------------------------------------------*/
{
   if ( PLA_Error_check( ) )
      PLA_Invert_check( alpha );
   
   if ( PLA_Obj_is_owner( alpha ) )
   {
      switch ( PLA_Obj_datatype( alpha ) )
      {
	 case PLA_FLOAT:
	 {
            float *buff = PLA_FLOAT_PTR( alpha );
	    
	    *buff = 1.0 / *buff;
	    
            break;
	 }
	 case PLA_DOUBLE:
	 {
            double *buff = PLA_DOUBLE_PTR( alpha );
	    
	    *buff = 1.0 / *buff;
	    
            break;
	 }
	 case PLA_COMPLEX:
	 {
	    scomplex *buff = PLA_COMPLEX_PTR( alpha );
	    float     temp;
	    
	    temp =  1.0 / ( buff->real * buff->real +
			    buff->imag * buff->imag );
	    
	    buff->real = buff->real *  temp;
	    buff->imag = buff->imag * -temp;
	    
	    break;
	 }
	 case PLA_DOUBLE_COMPLEX:
	 {
	    dcomplex *buff = PLA_DOUBLE_COMPLEX_PTR( alpha );
	    double    temp;
	    
	    temp =  1.0 / ( buff->real * buff->real +
			    buff->imag * buff->imag );
	    
	    buff->real = buff->real *  temp;
	    buff->imag = buff->imag * -temp;
	    
	    break;
	 }
	 case PLA_CONSTANT:
	    PLA_Abort( "Cannot change constant", __LINE__, __FILE__ );
	    break;
	 default:
	    PLA_Abort( "Datatype not yet implemented", __LINE__, __FILE__ );
	    break;
      }
   }  
   
   return PLA_SUCCESS;
}
