/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


float PLA_random_float( )
/*----------------------------------------------------------------------------

Purpose :  Create a random float

Return :   random float

----------------------------------------------------------------------------*/
{
   return ( (float) drand48( ) * 2.0 - 1.0 );
}


double PLA_random_double( )
/*----------------------------------------------------------------------------

Purpose :  Create a random double

Return :   random double

----------------------------------------------------------------------------*/
{
   return ( (double) drand48( ) * 2.0 - 1.0 );
}


PLA_Error PLA_Random_matrix( PLA_Obj A )
/*----------------------------------------------------------------------------

Purpose :  Fill the object with random numbers

OUT     A      object

----------------------------------------------------------------------------*/
{
   int
      i, j,
      m, n,
      ldim;
   
   m = PLA_Obj_local_length ( A );
   n = PLA_Obj_local_width  ( A );
   ldim = PLA_Obj_local_ldim( A );
   
   switch ( PLA_Obj_datatype( A ) )
   {
      case PLA_FLOAT:
      {
         float *buff = PLA_FLOAT_PTR( A );
         
         for ( j = 0; j < n; j++ )
            for ( i = 0; i < m; i++ )
               buff[ j * ldim + i ] = PLA_random_float( );
         
         break;
      }
      case PLA_DOUBLE:
      {
         double *buff = PLA_DOUBLE_PTR( A );
         
         for ( j = 0; j < n; j++ )
            for ( i = 0; i < m; i++ )
               buff[ j * ldim + i ] = PLA_random_double( );
         
         break;
      }
      case PLA_COMPLEX:
      {
         scomplex *buff = PLA_COMPLEX_PTR( A );
         
         for ( j = 0; j < n; j++ )
            for ( i = 0; i < m; i++ )
            {
               buff[ j * ldim + i ].real = PLA_random_float( );
               buff[ j * ldim + i ].imag = PLA_random_float( );
            }
         
         break;
      }
      case PLA_DOUBLE_COMPLEX:
      {
         dcomplex *buff = PLA_DOUBLE_COMPLEX_PTR( A );
         
         for ( j = 0; j < n; j++ )
            for ( i = 0; i < m; i++ )
            {
               buff[ j * ldim + i ].real = PLA_random_double( );
               buff[ j * ldim + i ].imag = PLA_random_double( );
            }
         
         break;
      }     
      case PLA_CONSTANT:
         PLA_Abort( "Cannot change constant", __LINE__, __FILE__ );
         break;
      default:
         PLA_Abort( "Datatype not yet implemented", __LINE__, __FILE__ );
         break;
   }
   
   return PLA_SUCCESS;
}
