/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Apply_USU( FLA_Side side, FLA_Uplo uplo, FLA_Trans transa, FLA_Obj A, FLA_Obj S, FLA_Obj B )
{
  FLA_Error r_val;
  int nb_alg;

  // Extract the appropriate blocksize for the given datatype.
  nb_alg = FLA_Query_blocksize( FLA_Obj_datatype( A ), FLA_DIMENSION_MIN );

  // Invoke FLA_Apply_USU_internal().
  r_val = FLA_Apply_USU_internal( side, uplo, transa, A, S, B, nb_alg );

  return r_val;
}
  
void FLA_F2C( fla_apply_usu_f ) 
     ( F_INT *side, F_INT* uplo, F_INT *transa, F_INT *A, F_INT *S, F_INT *B, F_INT *IERROR )
{
  *IERROR = FLA_Apply_USU( *( ( int     * ) side   ),
                           *( ( int     * ) uplo   ), 
                           *( ( int     * ) transa ), 
                           *( ( FLA_Obj * ) A      ), 
                           *( ( FLA_Obj * ) S      ), 
                           *( ( FLA_Obj * ) B      ) ) ; 
}

