/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

extern fla_lq_t*  fla_lq_cntl;
extern fla_lq_t*  fla_lq_cntl2;

extern double     fla_lq_var1_in_to_ou_bsize_ratio;

FLA_Error FLA_LQ( FLA_Obj A, FLA_Obj t, FLA_Obj S )
{
  FLA_Error r_val;
  int       nb_alg, nb_alg_sm;
#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  FLA_Bool  check_params;

  // Check parameters.
  if ( FLA_Check_parameters() )
    FLA_LQ_check( A, t );

  // Temporarily disable parameter checking.
  check_params = FLA_Check_parameters_set( FALSE );
#endif

  // Inspect the length of S to get the implied blocksize.
  nb_alg = FLA_Obj_length( S );

  // Compute the blocksize for the inner variant.
  nb_alg_sm = ( int ) max( nb_alg * fla_lq_var1_in_to_ou_bsize_ratio, 1 );

  // Adjust the top parent control tree node based on the implied blocksize.
  FLA_Blocksize_set( FLA_Cntl_blocksize( fla_lq_cntl ),
                     nb_alg, nb_alg, nb_alg, nb_alg );

  // Addjust the child control tree node based on the implied blocksize.
  if ( FLA_Cntl_blocksize( FLA_Cntl_sub_lq( fla_lq_cntl ) ) != NULL )
    FLA_Blocksize_set( FLA_Cntl_blocksize( FLA_Cntl_sub_lq( fla_lq_cntl ) ),
                       nb_alg_sm, nb_alg_sm, nb_alg_sm, nb_alg_sm );

  // Invoke the blocked variant.
  r_val = FLA_LQ_blk_var1( A, t, S, fla_lq_cntl2 );

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  // Restore parameter checking to its previous status.
  FLA_Check_parameters_set( check_params );
#endif

  return r_val;
}

void FLA_F2C( fla_lq_f ) 
     ( F_INT *A, F_INT *t, F_INT *S, F_INT *IERROR )
{
  *IERROR = FLA_LQ( *( ( FLA_Obj * ) A ), 
                    *( ( FLA_Obj * ) t ), 
                    *( ( FLA_Obj * ) S ) );
}

