*
*  libflame
*  An object-based infrastructure for developing high-performance
*  dense linear algebra libraries.
*
*  Copyright (C) 2008, The University of Texas
*
*  libflame is free software; you can redistribute it and/or modify
*  it under the terms of the GNU Lesser General Public License as
*  published by the Free Software Foundation; either version 2.1 of
*  the License, or (at your option) any later version.
*
*  libflame is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public
*  License along with libflame; if you did not receive a copy, see
*  http://www.gnu.org/licenses/.
*
*  For more information, please contact us at flame@cs.utexas.edu or
*  send mail to:
*
*  Field G. Van Zee and/or
*  Robert A. van de Geijn
*  The University of Texas at Austin
*  Department of Computer Sciences
*  1 University Station C0500
*  Austin TX 78712
*
      SUBROUTINE SGEQRF( M, N, A, LDA, TAU, WORK, LWORK, INFO )
*
*  -- LAPACK routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     June 30, 1999
*
*     .. Scalar Arguments ..
      INTEGER            INFO, LDA, LWORK, M, N
*     ..
*     .. Array Arguments ..
      REAL               A( LDA, * ), TAU( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  SGEQRF computes a QR factorization of a real M-by-N matrix A:
*  A = Q * R.
*
*  Arguments
*  =========
*
*  M       (input) INTEGER
*          The number of rows of the matrix A.  M >= 0.
*
*  N       (input) INTEGER
*          The number of columns of the matrix A.  N >= 0.
*
*  A       (input/output) REAL array, dimension (LDA,N)
*          On entry, the M-by-N matrix A.
*          On exit, the elements on and above the diagonal of the array
*          contain the min(M,N)-by-N upper trapezoidal matrix R (R is
*          upper triangular if m >= n); the elements below the diagonal,
*          with the array TAU, represent the orthogonal matrix Q as a
*          product of min(m,n) elementary reflectors (see Further
*          Details).
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,M).
*
*  TAU     (output) REAL array, dimension (min(M,N))
*          The scalar factors of the elementary reflectors (see Further
*          Details).
*
*  WORK    (workspace/output) REAL array, dimension (LWORK)
*          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
*
*  LWORK   (input) INTEGER
*          The dimension of the array WORK.  LWORK >= max(1,N).
*          For optimum performance LWORK >= N*NB, where NB is
*          the optimal blocksize.
*
*          If LWORK = -1, then a workspace query is assumed; the routine
*          only calculates the optimal size of the WORK array, returns
*          this value as the first entry of the WORK array, and no error
*          message related to LWORK is issued by XERBLA.
*
*  INFO    (output) INTEGER
*          = 0:  successful exit
*          < 0:  if INFO = -i, the i-th argument had an illegal value
*
*  Further Details
*  ===============
*
*  The matrix Q is represented as a product of elementary reflectors
*
*     Q = H(1) H(2) . . . H(k), where k = min(m,n).
*
*  Each H(i) has the form
*
*     H(i) = I - tau * v * v'
*
*  where tau is a real scalar, and v is a real vector with
*  v(1:i-1) = 0 and v(i) = 1; v(i+1:m) is stored on exit in A(i+1:m,i),
*  and tau in TAU(i).
*
*  =====================================================================
*
*     .. Local Scalars ..
      LOGICAL            LQUERY
      INTEGER            IINFO, IWS, K, LDWORK, LWKOPT, NB,
     $                   NBMIN, NX
*     ..
*     .. External Subroutines ..
      EXTERNAL           SGEQR2, SLARFB, SLARFT, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN
*     ..
*     .. External Functions ..
      INTEGER            ILAENV
      EXTERNAL           ILAENV
*     ..
*     .. Declare FLAME related stuff
      include "FLAMEf.h"
*
      INTEGER            FLA_QUERY_BLOCKSIZE_F
      EXTERNAL           FLA_QUERY_BLOCKSIZE_F
*
      INTEGER            A_obj( FLA_OBJ_SIZE ),
     $                   t_obj( FLA_OBJ_SIZE ),
     $                   S_obj( FLA_OBJ_SIZE ),
     $                   dtype
      PARAMETER        ( dtype = FLA_FLOAT )
*     ..
*     .. Executable Statements ..
*
*     Initialize FLAME.
*
      CALL FLA_INIT_F( IINFO )
*
*     Test the input arguments
*
      INFO = 0
c     NB = ILAENV( 1, 'SGEQRF', ' ', M, N, -1, -1 )
      NB = FLA_QUERY_BLOCKSIZE_F( dtype, FLA_DIMENSION_MIN ) / 2

      LWKOPT = N*NB
      WORK( 1 ) = LWKOPT
      LQUERY = ( LWORK.EQ.-1 )
      IF( M.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( LDA.LT.MAX( 1, M ) ) THEN
         INFO = -4
      ELSE IF( LWORK.LT.MAX( 1, N ) .AND. .NOT.LQUERY ) THEN
         INFO = -7
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'SGEQRF', -INFO )
         RETURN
      ELSE IF( LQUERY ) THEN
         RETURN
      END IF
*
*     Quick return if possible
*
      K = MIN( M, N )
      IF( K.EQ.0 ) THEN
         WORK( 1 ) = 1
         RETURN
      END IF
*
      NBMIN = 2
      NX = 0
      IWS = N
      IF( NB.GT.1 .AND. NB.LT.K ) THEN
*
*        Determine when to cross over from blocked to unblocked code.
*
c        NX = MAX( 0, ILAENV( 3, 'SGEQRF', ' ', M, N, -1, -1 ) )
         NX = 0
         IF( NX.LT.K ) THEN
*
*           Determine if workspace is large enough for blocked code.
*
            LDWORK = N
            IWS = LDWORK*NB
            IF( LWORK.LT.IWS ) THEN
*
*              Not enough workspace to use optimal NB:  reduce NB and
*              determine the minimum value of NB.
*
               NB = LWORK / LDWORK
               NBMIN = MAX( 2, ILAENV( 2, 'SGEQRF', ' ', M, N, -1,
     $                 -1 ) )
            END IF
         END IF
      END IF
*
      IF ( .FALSE. ) THEN
*
*        Use unblocked LAPACK code.
*
         CALL SGEQR2( M, N, A, LDA, TAU, WORK, IINFO )
*
      ELSE
*
*        Use blocked FLAME code.
*
*        Create base object for A_obj and attach matrix buffer.
*
         CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, M, N, A_obj,
     $                                     IINFO )
         CALL FLA_OBJ_ATTACH_BUFF_F( A, LDA, A_obj, IINFO )
*
*        Create base object for S_obj with nb-by-n buffer.
*
         CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, NB, N, S_obj,
     $                                     IINFO )
         CALL FLA_OBJ_ATTACH_BUFF_F( WORK, NB, S_obj, IINFO )
*
*        Create base object for t_obj and attach vector buffer.
*
         CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, min(M,N), 1, t_obj,
     $                                     IINFO )
         CALL FLA_OBJ_ATTACH_BUFF_F( TAU, min(M,N), t_obj, IINFO )
*
*        Call the FLAME QR code.
*
         CALL FLA_QR_F( A_obj, t_obj, S_obj, IINFO )
*
*        Free A_obj base object (but leave A buffer alone).
*
         CALL FLA_OBJ_FREE_W_OUT_BUFF_F( A_obj, IINFO )
*
*        Free S_obj base object (but leave WORK buffer alone).
*
         CALL FLA_OBJ_FREE_W_OUT_BUFF_F( S_obj, IINFO )
*
*        Free t_obj base object (but leave TAU buffer alone).
*
         CALL FLA_OBJ_FREE_W_OUT_BUFF_F( t_obj, IINFO )
*
      ENDIF
*
      RETURN
*
*     End of SGEQRF
*
      END
