/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Accum_S_unb_var1( FLA_Obj U, FLA_Obj s, FLA_Obj S )
{
  FLA_Obj UTL,   UTR,      U00,  u01,       U02, 
          UBL,   UBR,      u10t, upsilon11, u12t,
                           U20,  u21,       U22;

  FLA_Obj sT,              s0,
          sB,              sigma1,
                           s2;

  FLA_Obj STL,   STR,      S00,  s01,     S02, 
          SBL,   SBR,      s10t, sigma11, s12t,
                           S20,  s21,     S22;

  FLA_Part_2x2( U,    &UTL, &UTR,
                      &UBL, &UBR,     0, 0, FLA_TL );

  FLA_Part_2x1( s,    &sT, 
                      &sB,            0, FLA_TOP );

  FLA_Part_2x2( S,    &STL, &STR,
                      &SBL, &SBR,     0, 0, FLA_TL );

  while ( FLA_Obj_width( UTL ) != FLA_Obj_width( U ) ){

    FLA_Repart_2x2_to_3x3( UTL, /**/ UTR,       &U00,  /**/ &u01,       &U02,
                        /* ************* */   /* **************************** */
                                                &u10t, /**/ &upsilon11, &u12t,
                           UBL, /**/ UBR,       &U20,  /**/ &u21,       &U22,
                           1, 1, FLA_BR );

    FLA_Repart_2x1_to_3x1( sT,                &s0, 
                        /* ** */            /* ****** */
                                              &sigma1, 
                           sB,                &s2,        1, FLA_BOTTOM );

    FLA_Repart_2x2_to_3x3( STL, /**/ STR,       &S00,  /**/ &s01,     &S02,
                        /* ************* */   /* ************************** */
                                                &s10t, /**/ &sigma11, &s12t,
                           SBL, /**/ SBR,       &S20,  /**/ &s21,     &S22,
                           1, 1, FLA_BR );

    /*------------------------------------------------------------*/

    /* Update S
    sigma11 = sigma1;
    s01  = - triu( S00 ) * ( u10t' + U20' * u21 ) * sigma11;
    */

    FLA_Copy( sigma1, sigma11 );
    FLA_Copy_t( FLA_TRANSPOSE, u10t, s01 );
    FLA_Gemv( FLA_TRANSPOSE, FLA_ONE, U20, u21, FLA_ONE, s01 );
    FLA_Trmv( FLA_UPPER_TRIANGULAR, FLA_NO_TRANSPOSE, FLA_NONUNIT_DIAG,
              S00, s01 );
    FLA_Scal( sigma11, s01 );
    FLA_Scal( FLA_MINUS_ONE, s01 );

    /*------------------------------------------------------------*/

    FLA_Cont_with_3x3_to_2x2( &UTL, /**/ &UTR,       U00,  u01,       /**/ U02,
                                                     u10t, upsilon11, /**/ u12t,
                            /* ************** */  /* ************************** */
                              &UBL, /**/ &UBR,       U20,  u21,       /**/ U22,
                              FLA_TL );

    FLA_Cont_with_3x1_to_2x1( &sT,                s0, 
                                                  sigma1, 
                            /* ** */           /* ****** */
                              &sB,                s2,     FLA_TOP );

    FLA_Cont_with_3x3_to_2x2( &STL, /**/ &STR,       S00,  s01,     /**/ S02,
                                                     s10t, sigma11, /**/ s12t,
                            /* ************** */  /* ************************ */
                              &SBL, /**/ &SBR,       S20,  s21,     /**/ S22,
                              FLA_TL );

  }

  return FLA_SUCCESS;
}
