/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/
#include "FLAME.h"

#define FLA_ALG_REFERENCE 0
#define FLA_ALG_BLOCKED   1
#define FLA_ALG_UNBLOCKED 2


FLA_Error REF_QR_UT( FLA_Obj A, FLA_Obj t );
void time_QR(
                  int variant, int type, int nrepeats, int m, int n, int nb_alg,
                  FLA_Obj A, FLA_Obj A_ref, FLA_Obj t, FLA_Obj S,
                  double *dtime, double *diff, double *gflops );


void time_QR(
                  int variant, int type, int nrepeats, int m, int n, int nb_alg,
                  FLA_Obj A, FLA_Obj A_ref, FLA_Obj t, FLA_Obj S,
                  double *dtime, double *diff, double *gflops )
{
  int
    irep;
  int nb_alg_sm;

  double
    dtime_old = 1.0e9;

  FLA_Obj
    A_old, x, B, nrm_B;
  double* nrm_B_p;

  fla_blocksize_t*
    bp_la;
  fla_blocksize_t*
    bp_sm;
  fla_qr_t*
    cntl_qr_ut;
  fla_qr_t*
    cntl_qr_ut_var;
  fla_qr_t*
    cntl_qr_ut_lapack;
  fla_trmm_t*
    cntl_trmm_blas;
  fla_gemm_t*
    cntl_gemm_blas;

  nb_alg_sm         = max( nb_alg/4, 1 );
  bp_la             = FLA_Blocksize_create( nb_alg, nb_alg, nb_alg, nb_alg );
  bp_sm             = FLA_Blocksize_create( nb_alg_sm, nb_alg_sm, nb_alg_sm, nb_alg_sm );
  cntl_trmm_blas    = FLA_Cntl_trmm_obj_create( FLA_FLAT, FLA_SUBPROBLEM, NULL, NULL, NULL );
  cntl_gemm_blas    = FLA_Cntl_gemm_obj_create( FLA_FLAT, FLA_SUBPROBLEM, NULL, NULL );
  cntl_qr_ut_lapack = FLA_Cntl_qr_obj_create( FLA_FLAT, FLA_SUBPROBLEM, NULL, NULL, NULL, NULL, NULL, NULL, NULL );
  cntl_qr_ut_var    = FLA_Cntl_qr_obj_create( FLA_FLAT, variant, bp_sm, cntl_qr_ut_lapack, cntl_trmm_blas, cntl_trmm_blas, cntl_trmm_blas, cntl_gemm_blas, cntl_gemm_blas );
  cntl_qr_ut        = FLA_Cntl_qr_obj_create( FLA_FLAT, variant, bp_la, cntl_qr_ut_var, cntl_trmm_blas, cntl_trmm_blas, cntl_trmm_blas, cntl_gemm_blas, cntl_gemm_blas );


  FLA_Obj_create_conf_to( FLA_NO_TRANSPOSE, A, &A_old );

  FLA_Copy( A, A_old );


  for ( irep = 0 ; irep < nrepeats; irep++ ){
    FLA_Copy( A_old, A );

    *dtime = FLA_Clock();

    switch( variant ){
    case 0:
      /* Time reference implementation */
      //REF_QR_UT( A, t );
      FLA_QR_UT_unb_external( A, t, NULL );
      //REF_QR_UT_x( A, t, nb_alg );

      break;

    case 1:{

      // Time variant 1 
      switch( type ){
      case FLA_ALG_UNBLOCKED:
        //FLA_QR_UT_Accum_T_unb_var2( A, t, S );
        break;
      case FLA_ALG_BLOCKED:
        FLA_QR_UT_blk_var1( A, t, S, cntl_qr_ut );
        break;
      default:
        printf("trouble\n");
      }

      break;
    }

    }

    *dtime = FLA_Clock() - *dtime;
    dtime_old = min( *dtime, dtime_old );

/*
    FLA_Obj_create( FLA_DOUBLE, FLA_Obj_length( A ), 1, &B );
    FLA_Obj_create( FLA_DOUBLE, FLA_Obj_length( A ), 1, &x );
    FLA_Obj_create( FLA_DOUBLE, 1, 1, &nrm_B );

    FLA_Random_matrix( B );
    FLA_Copy( B, x );

    FLA_Apply_USU_llt_blk_var1( A, S, x, nb_alg );

    FLA_Trsv( FLA_UPPER_TRIANGULAR, FLA_NO_TRANSPOSE, FLA_NONUNIT_DIAG, A, x );
    FLA_Gemv( FLA_NO_TRANSPOSE, FLA_MINUS_ONE, A_old, x, FLA_ONE, B );

    FLA_Nrm2( B, nrm_B );
    nrm_B_p = FLA_DOUBLE_PTR(nrm_B);
    fprintf( stdout, "norm of B = %e\n", *nrm_B_p );

    FLA_Obj_free( &B );
    FLA_Obj_free( &x );
    FLA_Obj_free( &nrm_B );
*/

  }

  FLA_Cntl_obj_free( cntl_qr_ut );
  FLA_Cntl_obj_free( cntl_qr_ut_var );
  FLA_Cntl_obj_free( cntl_qr_ut_lapack );
  FLA_Cntl_obj_free( cntl_trmm_blas );
  FLA_Cntl_obj_free( cntl_gemm_blas );
  FLA_Blocksize_free( bp_sm );
  FLA_Blocksize_free( bp_la );

  if ( variant == 0 ){
    FLA_Copy( A, A_ref );
    *diff = 0.0;
  }
  else{
    //    FLA_Obj_show( "A = ", A, "%lf ", " " );
    //    FLA_Obj_show( "Aref = ", A_ref, "%lf ", " " );
    *diff = FLA_Max_elemwise_diff( A, A_ref );
  }

  *gflops = 2.0 * n * n * (m - n/3.0) /
            dtime_old / 1e9;

  *dtime = dtime_old;

  FLA_Copy( A_old, A );

  FLA_Obj_free( &A_old );
}

