/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

extern fla_spdinv_t*    fla_spdinv_cntl;
extern fla_blocksize_t* fla_spdinv_size_cutoff;

FLA_Error FLA_SPDinv( FLA_Uplo uplo, FLA_Obj A )
{
  FLA_Datatype datatype;
  int          m_A, r_val = 0;
  int          FLA_SPDINV_CUTOFF;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  FLA_Bool     check_params;

  // Check parameters.
  if ( FLA_Check_parameters() )
    FLA_SPDinv_check( uplo, A );

  // Temporarily disable parameter checking.
  check_params = FLA_Check_parameters_set( FALSE );
#endif

  // Determine the datatype of the operation.
  datatype = FLA_Obj_datatype( A );

  // Extract the appropriate blocksize for the given datatype.
  FLA_SPDINV_CUTOFF = FLA_Blocksize_extract( datatype, fla_spdinv_size_cutoff );

  // Determine the dimension of A.
  m_A = FLA_Obj_length( A );

  // Invoke FLA_SPDinv_internal() with an appropriate control tree.
  if      ( m_A <= FLA_SPDINV_CUTOFF )
  {
    r_val = FLA_SPDinv_blk_external( uplo, A );
  }
  else if ( FLA_SPDINV_CUTOFF < m_A )
  {
    r_val = FLA_SPDinv_internal( uplo, A, fla_spdinv_cntl );
  }

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  // Restore parameter checking to its previous status.
  FLA_Check_parameters_set( check_params );
#endif

  return r_val;
}

