*
*  libflame
*  An object-based infrastructure for developing high-performance
*  dense linear algebra libraries.
*
*  Copyright (C) 2008, The University of Texas
*
*  libflame is free software; you can redistribute it and/or modify
*  it under the terms of the GNU Lesser General Public License as
*  published by the Free Software Foundation; either version 2.1 of
*  the License, or (at your option) any later version.
*
*  libflame is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public
*  License along with libflame; if you did not receive a copy, see
*  http://www.gnu.org/licenses/.
*
*  For more information, please contact us at flame@cs.utexas.edu or
*  send mail to:
*
*  Field G. Van Zee and/or
*  Robert A. van de Geijn
*  The University of Texas at Austin
*  Department of Computer Sciences
*  1 University Station C0500
*  Austin TX 78712
*
      SUBROUTINE DLAUUM( UPLO, N, A, LDA, INFO )
*
*  -- LAPACK auxiliary routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     February 29, 1992
*
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            INFO, LDA, N
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * )
*     ..
*
*  Purpose
*  =======
*
*  DLAUUM computes the product U * U' or L' * L, where the triangular
*  factor U or L is stored in the upper or lower triangular part of
*  the array A.
*
*  If UPLO = 'U' or 'u' then the upper triangle of the result is stored,
*  overwriting the factor U in A.
*  If UPLO = 'L' or 'l' then the lower triangle of the result is stored,
*  overwriting the factor L in A.
*
*  This is the blocked form of the algorithm, calling Level 3 BLAS.
*
*  Arguments
*  =========
*
*  UPLO    (input) CHARACTER*1
*          Specifies whether the triangular factor stored in the array A
*          is upper or lower triangular:
*          = 'U':  Upper triangular
*          = 'L':  Lower triangular
*
*  N       (input) INTEGER
*          The order of the triangular factor U or L.  N >= 0.
*
*  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
*          On entry, the triangular factor U or L.
*          On exit, if UPLO = 'U', the upper triangle of A is
*          overwritten with the upper triangle of the product U * U';
*          if UPLO = 'L', the lower triangle of A is overwritten with
*          the lower triangle of the product L' * L.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,N).
*
*  INFO    (output) INTEGER
*          = 0: successful exit
*          < 0: if INFO = -k, the k-th argument had an illegal value
*
*  =====================================================================
*
*     .. Parameters ..
*     ..
*     .. Local Scalars ..
      LOGICAL            UPPER
      INTEGER            IINFO
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL           DLAUU2, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     .. Declare FLAME related stuff
      include "FLAMEf.h"
*
      INTEGER            A_obj( FLA_OBJ_SIZE ),
     $                   dtype
      PARAMETER        ( dtype = FLA_DOUBLE )
*     ..
*     .. Executable Statements ..
*
*     Initialize FLAME
*
      CALL FLA_INIT_F( IINFO )
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
      UPPER = LSAME( UPLO, 'U' )
      IF( .NOT.UPPER .AND. .NOT.LSAME( UPLO, 'L' ) ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( LDA.LT.MAX( 1, N ) ) THEN
         INFO = -4
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DLAUUM', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( N.EQ.0 )
     $   RETURN
*
      IF( .FALSE. ) THEN
*
*        Use unblocked code
*
         CALL DLAUU2( UPLO, N, A, LDA, INFO )
*
      ELSE
*
*        Use blocked FLAME code.
*
*        Create base object for A_obj and attach matrix buffer.
*
         CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, N, N, A_obj,
     $                                     IINFO )
         CALL FLA_OBJ_ATTACH_BUFF_F( A, LDA, A_obj, IINFO )
*
         IF( UPPER ) THEN
*
*           Compute the matrix product A = U'*U.
*
            CALL FLA_TTMM_F( FLA_UPPER_TRIANGULAR, A_obj, IINFO )
*
         ELSE
*
*           Compute the matrix product A = L*L'.
*
            CALL FLA_TTMM_F( FLA_LOWER_TRIANGULAR, A_obj, IINFO )
*
         END IF
*
*        Free A_obj base object (but leave A buffer alone).
*
         CALL FLA_OBJ_FREE_W_OUT_BUFF_F( A_obj, IINFO )
*
      END IF
*
      RETURN
*
*     End of DLAUUM
*
      END
