*
*  libflame
*  An object-based infrastructure for developing high-performance
*  dense linear algebra libraries.
*
*  Copyright (C) 2008, The University of Texas
*
*  libflame is free software; you can redistribute it and/or modify
*  it under the terms of the GNU Lesser General Public License as
*  published by the Free Software Foundation; either version 2.1 of
*  the License, or (at your option) any later version.
*
*  libflame is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public
*  License along with libflame; if you did not receive a copy, see
*  http://www.gnu.org/licenses/.
*
*  For more information, please contact us at flame@cs.utexas.edu or
*  send mail to:
*
*  Field G. Van Zee and/or
*  Robert A. van de Geijn
*  The University of Texas at Austin
*  Department of Computer Sciences
*  1 University Station C0500
*  Austin TX 78712
*
      SUBROUTINE DGELS( TRANS, M, N, NRHS, A, LDA, B, LDB, WORK, LWORK,
     $                  INFO )
*
*  -- LAPACK driver routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     June 30, 1999
*
*     .. Scalar Arguments ..
      CHARACTER          TRANS
      INTEGER            INFO, LDA, LDB, LWORK, M, N, NRHS
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), B( LDB, * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  DGELS solves overdetermined or underdetermined real linear systems
*  involving an M-by-N matrix A, or its transpose, using a QR or LQ
*  factorization of A.  It is assumed that A has full rank.
*
*  The following options are provided:
*
*  1. If TRANS = 'N' and m >= n:  find the least squares solution of
*     an overdetermined system, i.e., solve the least squares problem
*                  minimize || B - A*X ||.
*
*  2. If TRANS = 'N' and m < n:  find the minimum norm solution of
*     an underdetermined system A * X = B.
*
*  3. If TRANS = 'T' and m >= n:  find the minimum norm solution of
*     an undetermined system A**T * X = B.
*
*  4. If TRANS = 'T' and m < n:  find the least squares solution of
*     an overdetermined system, i.e., solve the least squares problem
*                  minimize || B - A**T * X ||.
*
*  Several right hand side vectors b and solution vectors x can be
*  handled in a single call; they are stored as the columns of the
*  M-by-NRHS right hand side matrix B and the N-by-NRHS solution
*  matrix X.
*
*  Arguments
*  =========
*
*  TRANS   (input) CHARACTER
*          = 'N': the linear system involves A;
*          = 'T': the linear system involves A**T.
*
*  M       (input) INTEGER
*          The number of rows of the matrix A.  M >= 0.
*
*  N       (input) INTEGER
*          The number of columns of the matrix A.  N >= 0.
*
*  NRHS    (input) INTEGER
*          The number of right hand sides, i.e., the number of
*          columns of the matrices B and X. NRHS >=0.
*
*  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
*          On entry, the M-by-N matrix A.
*          On exit,
*            if M >= N, A is overwritten by details of its QR
*                       factorization as returned by DGEQRF;
*            if M <  N, A is overwritten by details of its LQ
*                       factorization as returned by DGELQF.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,M).
*
*  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
*          On entry, the matrix B of right hand side vectors, stored
*          columnwise; B is M-by-NRHS if TRANS = 'N', or N-by-NRHS
*          if TRANS = 'T'.
*          On exit, B is overwritten by the solution vectors, stored
*          columnwise:
*          if TRANS = 'N' and m >= n, rows 1 to n of B contain the least
*          squares solution vectors; the residual sum of squares for the
*          solution in each column is given by the sum of squares of
*          elements N+1 to M in that column;
*          if TRANS = 'N' and m < n, rows 1 to N of B contain the
*          minimum norm solution vectors;
*          if TRANS = 'T' and m >= n, rows 1 to M of B contain the
*          minimum norm solution vectors;
*          if TRANS = 'T' and m < n, rows 1 to M of B contain the
*          least squares solution vectors; the residual sum of squares
*          for the solution in each column is given by the sum of
*          squares of elements M+1 to N in that column.
*
*  LDB     (input) INTEGER
*          The leading dimension of the array B. LDB >= MAX(1,M,N).
*
*  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
*          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
*
*  LWORK   (input) INTEGER
*          The dimension of the array WORK.
*          LWORK >= max( 1, MN + max( MN, NRHS ) ).
*          For optimal performance,
*          LWORK >= max( 1, MN + max( MN, NRHS )*NB ).
*          where MN = min(M,N) and NB is the optimum block size.
*
*          If LWORK = -1, then a workspace query is assumed; the routine
*          only calculates the optimal size of the WORK array, returns
*          this value as the first entry of the WORK array, and no error
*          message related to LWORK is issued by XERBLA.
*
*  INFO    (output) INTEGER
*          = 0:  successful exit
*          < 0:  if INFO = -i, the i-th argument had an illegal value
*
* *********************************************************************
* 
* Modified to become an interface to the FLAME QR factorization
* Aug. 5, 2004
* Robert van de Geijn
* rvdg@cs.utexas.edu
*
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION   ZERO_LA, ONE_LA
      PARAMETER          ( ZERO_LA = 0.0D0, ONE_LA = 1.0D0 )
*     ..
*     .. Local Scalars ..
      LOGICAL            LQUERY, TPSD
      INTEGER            BROW, I, IASCL, IBSCL, J, MN, NB, SCLLEN, 
     $                   WSIZE, NB_ALG, IINFO
      DOUBLE PRECISION   ANRM, BIGNUM, BNRM, SMLNUM
*     ..
*     .. Local Arrays ..
      DOUBLE PRECISION   RWORK( 1 )
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            ILAENV
      DOUBLE PRECISION   DLAMCH, DLANGE
      EXTERNAL           LSAME, ILAENV, DLAMCH, DLANGE
*     ..
*     .. External Subroutines ..
      EXTERNAL           DGELQF, DGEQRF, DLASCL, DLASET, DORMLQ, DORMQR,
     $                   DTRSM, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          DBLE, MAX, MIN
*     ..
*     ..       
*     .. Declare FLAME related stuff
      include "FLAMEf.h"
*
      INTEGER            A_obj( FLA_OBJ_SIZE ),
     $                   t_obj( FLA_OBJ_SIZE ),
     $                   B_obj( FLA_OBJ_SIZE ),
     $                   S_obj( FLA_OBJ_SIZE ),
     $                   dtype
      PARAMETER        ( dtype = FLA_DOUBLE )
*     ..
*     .. Executable Statements ..
*
      CALL FLA_INIT_F( IINFO )
*
*     Test the input arguments.
*
      INFO = 0
      TPSD = .FALSE.
      WSIZE = 0
      MN = MIN( M, N )
      LQUERY = ( LWORK.EQ.-1 )
      IF( .NOT.( LSAME( TRANS, 'N' ) .OR. LSAME( TRANS, 'T' ) ) ) THEN
         INFO = -1
      ELSE IF( M.LT.0 ) THEN
         INFO = -2
      ELSE IF( N.LT.0 ) THEN
         INFO = -3
      ELSE IF( NRHS.LT.0 ) THEN
         INFO = -4
      ELSE IF( LDA.LT.MAX( 1, M ) ) THEN
         INFO = -6
      ELSE IF( LDB.LT.MAX( 1, M, N ) ) THEN
         INFO = -8
      ELSE IF( LWORK.LT.MAX( 1, MN+MAX( MN, NRHS ) ) .AND. .NOT.LQUERY )
     $          THEN
         INFO = -10
      END IF
*
*     Figure out optimal block size
*
      IF( INFO.EQ.0 .OR. INFO.EQ.-10 ) THEN
*
         TPSD = .TRUE.
         IF( LSAME( TRANS, 'N' ) )
     $      TPSD = .FALSE.
*
         IF( M.GE.N ) THEN
            NB = ILAENV( 1, 'DGEQRF', ' ', M, N, -1, -1 )
            IF( TPSD ) THEN
               NB = MAX( NB, ILAENV( 1, 'DORMQR', 'LN', M, NRHS, N,
     $              -1 ) )
            ELSE
               NB = MAX( NB, ILAENV( 1, 'DORMQR', 'LT', M, NRHS, N,
     $              -1 ) )
            END IF
         ELSE
            NB = ILAENV( 1, 'DGELQF', ' ', M, N, -1, -1 )
            IF( TPSD ) THEN
               NB = MAX( NB, ILAENV( 1, 'DORMLQ', 'LT', N, NRHS, M,
     $              -1 ) )
            ELSE
               NB = MAX( NB, ILAENV( 1, 'DORMLQ', 'LN', N, NRHS, M,
     $              -1 ) )
            END IF
         END IF
*
         WSIZE = MAX( 1, MN+MAX( MN, NRHS )*NB )
         WORK( 1 ) = DBLE( WSIZE )
*
      END IF
*
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DGELS ', -INFO )
         RETURN
      ELSE IF( LQUERY ) THEN
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( MIN( M, N, NRHS ).EQ.0 ) THEN
         CALL DLASET( 'Full', MAX( M, N ), NRHS, ZERO_LA, ZERO_LA,
     $        B, LDB )
         RETURN
      END IF
*
*     Get machine parameters
*
      SMLNUM = DLAMCH( 'S' ) / DLAMCH( 'P' )
      BIGNUM = ONE_LA / SMLNUM
      CALL DLABAD( SMLNUM, BIGNUM )
*
*     Scale A, B if max element outside range [SMLNUM,BIGNUM]
*
      ANRM = DLANGE( 'M', M, N, A, LDA, RWORK )
      IASCL = 0
      IF( ANRM.GT.ZERO_LA .AND. ANRM.LT.SMLNUM ) THEN
*
*        Scale matrix norm up to SMLNUM
*
         CALL DLASCL( 'G', 0, 0, ANRM, SMLNUM, M, N, A, LDA, INFO )
         IASCL = 1
      ELSE IF( ANRM.GT.BIGNUM ) THEN
*
*        Scale matrix norm down to BIGNUM
*
         CALL DLASCL( 'G', 0, 0, ANRM, BIGNUM, M, N, A, LDA, INFO )
         IASCL = 2
      ELSE IF( ANRM.EQ.ZERO_LA ) THEN
*
*        Matrix all zero. Return zero solution.
*
         CALL DLASET( 'F', MAX( M, N ), NRHS, ZERO_LA, ZERO_LA, 
     $        B, LDB )
         GO TO 50
      END IF
*
      BROW = M
      IF( TPSD )
     $   BROW = N
      BNRM = DLANGE( 'M', BROW, NRHS, B, LDB, RWORK )
      IBSCL = 0
      IF( BNRM.GT.ZERO_LA .AND. BNRM.LT.SMLNUM ) THEN
*
*        Scale matrix norm up to SMLNUM
*
         CALL DLASCL( 'G', 0, 0, BNRM, SMLNUM, BROW, NRHS, B, LDB,
     $                INFO )
         IBSCL = 1
      ELSE IF( BNRM.GT.BIGNUM ) THEN
*
*        Scale matrix norm down to BIGNUM
*
         CALL DLASCL( 'G', 0, 0, BNRM, BIGNUM, BROW, NRHS, B, LDB,
     $                INFO )
         IBSCL = 2
      END IF
*
      IF( M.GE.N ) THEN
*
*        compute QR factorization of A
*
************************************************************************
c        CALL DGEQRF( M, N, A, LDA, WORK( 1 ), WORK( MN+1 ), LWORK-MN,
c    $                INFO )
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*
*        Determine the optimal block size
*
         NB_ALG = 100
*
*        Create base object for A_obj and attach matrix buffer.
*
         CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, M, N, 
     $                                     A_obj, IINFO )
         CALL FLA_OBJ_ATTACH_BUFF_F( A, LDA, A_obj, IINFO )
*
*        Create base object for S_obj with nb_alg-by-n buffer.
*
         CALL FLA_OBJ_CREATE_F( dtype, NB_ALG, N, S_obj,
     $                          IINFO )
*
*        Create base object for t_obj and attach WORK buffer.
*
         CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, MN, 1, 
     $                                     t_obj, IINFO )
         CALL FLA_OBJ_ATTACH_BUFF_F( WORK( 1 ), MN, t_obj, IINFO )
*
*        Call the FLAME QR code.
*
         CALL FLA_QR_F( A_obj, t_obj, S_obj, IINFO )
*
************************************************************************
*
*        workspace at least N, optimally N*NB
*
         IF( .NOT.TPSD ) THEN
*
*           Least-Squares Problem min || A * X - B ||
*
*           B(1:M,1:NRHS) := Q' * B(1:M,1:NRHS)
*
************************************************************************
c           CALL DORMQR( 'Left', 'Transpose', M, NRHS, N, A, LDA,
c    $                   WORK( 1 ), B, LDB, WORK( MN+1 ), LWORK-MN,
c    $                   INFO )
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*
*           Create base object for B_obj and attach matrix buffer.
*
            CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, M, NRHS, 
     $                                        B_obj, IINFO )
            CALL FLA_OBJ_ATTACH_BUFF_F( B, LDB, B_obj, IINFO )
*
*           Call FLA_Apply_USU().
*
            CALL FLA_APPLY_USU_LLT_F( A_obj, S_obj, B_obj, NB_ALG,
     $                                IINFO )
*
*           Free B_obj base object (but leave B buffer alone).
*
            CALL FLA_OBJ_FREE_W_OUT_BUFF_F( B_obj, IINFO )
*
************************************************************************
*
*           workspace at least NRHS, optimally NRHS*NB
*
*           B(1:N,1:NRHS) := inv(R) * B(1:N,1:NRHS)
*
            CALL DTRSM( 'Left', 'Upper', 'No transpose', 'Non-unit', N,
     $                  NRHS, ONE_LA, A, LDA, B, LDB )
*
            SCLLEN = N
*
         ELSE
*
*           Overdetermined system of equations A' * X = B
*
*           B(1:N,1:NRHS) := inv(R') * B(1:N,1:NRHS)
*
            CALL DTRSM( 'Left', 'Upper', 'Transpose', 'Non-unit', N,
     $                  NRHS, ONE_LA, A, LDA, B, LDB )
*
*           B(N+1:M,1:NRHS) = ZERO_LA
*
            DO 20 J = 1, NRHS
               DO 10 I = N + 1, M
                  B( I, J ) = ZERO_LA
   10          CONTINUE
   20       CONTINUE

*
*           B(1:M,1:NRHS) := Q(1:N,:) * B(1:N,1:NRHS)
*
************************************************************************
c           CALL DORMQR( 'Left', 'No transpose', M, NRHS, N, A, LDA,
c    $                   WORK( 1 ), B, LDB, WORK( MN+1 ), LWORK-MN,
c    $                   INFO )
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*
*           Create base object for B_obj and attach matrix buffer.
*
            CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, N, NRHS, 
     $                                        B_obj, IINFO )
            CALL FLA_OBJ_ATTACH_BUFF_F( B, LDB, B_obj, IINFO )
*
*           Call FLA_Apply_USU().
*
            CALL FLA_APPLY_USU_LLN_F( A_obj, S_obj, B_obj, NB_ALG,
     $                                IINFO )
*
*           Free B_obj base object (but leave B buffer alone).
*
            CALL FLA_OBJ_FREE_W_OUT_BUFF_F( B_obj, IINFO )
*
************************************************************************
*
*           workspace at least NRHS, optimally NRHS*NB
*
            SCLLEN = M
*
         END IF
*
*        Free base object and buffer for S_obj.
*
         CALL FLA_OBJ_FREE_F( S_obj, IINFO )
*
*        Free A_obj base object (but leave A buffer alone).
*
         CALL FLA_OBJ_FREE_W_OUT_BUFF_F( A_obj, IINFO )
*
*        Free t_obj base object (but leave WORK buffer alone).
*
         CALL FLA_OBJ_FREE_W_OUT_BUFF_F( t_obj, IINFO )
*
      ELSE
*
*        Compute LQ factorization of A
*
************************************************************************
         CALL DGELQF( M, N, A, LDA, WORK( 1 ), WORK( MN+1 ), LWORK-MN,
     $                INFO )
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*
*        Determine the optimal block size
*
c        NB_ALG = 100
*
*        Create base object for A_obj and attach matrix buffer.
*
c        CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, M, N, 
c    $                                     A_obj, IINFO )
c        CALL FLA_OBJ_ATTACH_BUFF_F( A, LDA, A_obj, IINFO )
*
*        Create base object for S_obj with nb_alg-by-m buffer.
*
c        CALL FLA_OBJ_CREATE_F( dtype, NB_ALG, M, S_obj,
c    $                          IINFO )
*
*        Create base object for t_obj and attach WORK buffer.
*
c        CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, MN, 1, 
c    $                                     t_obj, IINFO )
c        CALL FLA_OBJ_ATTACH_BUFF_F( WORK( 1 ), MN, t_obj, IINFO )
*
*        Call the FLAME LQ code.
*
c        CALL FLA_LQ_F( A_obj, t_obj, S_obj, IINFO )
*
************************************************************************
*
*        workspace at least M, optimally M*NB.
*
         IF( .NOT.TPSD ) THEN
*
*           underdetermined system of equations A * X = B
*
*           B(1:M,1:NRHS) := inv(L) * B(1:M,1:NRHS)
*
            CALL DTRSM( 'Left', 'Lower', 'No transpose', 'Non-unit', M,
     $                  NRHS, ONE_LA, A, LDA, B, LDB )
*
*           B(M+1:N,1:NRHS) = 0
*
            DO 40 J = 1, NRHS
               DO 30 I = M + 1, N
                  B( I, J ) = ZERO_LA
   30          CONTINUE
   40       CONTINUE
*
*           B(1:N,1:NRHS) := Q(1:N,:)' * B(1:M,1:NRHS)
*
************************************************************************
            CALL DORMLQ( 'Left', 'Transpose', N, NRHS, M, A, LDA,
     $                   WORK( 1 ), B, LDB, WORK( MN+1 ), LWORK-MN,
     $                   INFO )
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*
*           Create base object for B_obj and attach matrix buffer.
*
c           CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, M, NRHS, 
c    $                                        B_obj, IINFO )
c           CALL FLA_OBJ_ATTACH_BUFF_F( B, LDB, B_obj, IINFO )
*
*           Call FLA_Apply_USU().
*
c           CALL FLA_APPLY_USU_LLT_F( A_obj, S_obj, B_obj, NB_ALG,
c    $                                IINFO )
*
*           Free B_obj base object (but leave B buffer alone).
*
c           CALL FLA_OBJ_FREE_W_OUT_BUFF_F( B_obj, IINFO )
*
************************************************************************
*
*           workspace at least NRHS, optimally NRHS*NB
*
            SCLLEN = N
*
         ELSE
*
*           overdetermined system min || A' * X - B ||
*
*           B(1:N,1:NRHS) := Q * B(1:N,1:NRHS)
*
************************************************************************
            CALL DORMLQ( 'Left', 'No transpose', N, NRHS, M, A, LDA,
     $                   WORK( 1 ), B, LDB, WORK( MN+1 ), LWORK-MN,
     $                   INFO )
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*
*           Create base object for B_obj and attach matrix buffer.
*
c           CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, N, NRHS, 
c    $                                        B_obj, IINFO )
c           CALL FLA_OBJ_ATTACH_BUFF_F( B, LDB, B_obj, IINFO )
*
*           Call FLA_Apply_USU().
*
c           CALL FLA_APPLY_USU_LLN_F( A_obj, S_obj, B_obj, NB_ALG,
c    $                                IINFO )
*
*           Free B_obj base object (but leave B buffer alone).
*
c           CALL FLA_OBJ_FREE_W_OUT_BUFF_F( B_obj, IINFO )
*
************************************************************************
*
*           workspace at least NRHS, optimally NRHS*NB
*
*           B(1:M,1:NRHS) := inv(L') * B(1:M,1:NRHS)
*
            CALL DTRSM( 'Left', 'Lower', 'Transpose', 'Non-unit', M,
     $                  NRHS, ONE_LA, A, LDA, B, LDB )
*
            SCLLEN = M
*
         END IF
*
*        Free A_obj base object (but leave A buffer alone).
*
c        CALL FLA_OBJ_FREE_W_OUT_BUFF_F( A_obj, IINFO )
*
*        Free t_obj base object (but leave WORK buffer alone).
*
c        CALL FLA_OBJ_FREE_W_OUT_BUFF_F( t_obj, IINFO )
*
*        Free B_obj base object (but leave B buffer alone).
*
c        CALL FLA_OBJ_FREE_W_OUT_BUFF_F( B_obj, IINFO )
*
      END IF
*
*     Undo scaling
*
      IF( IASCL.EQ.1 ) THEN
         CALL DLASCL( 'G', 0, 0, ANRM, SMLNUM, SCLLEN, NRHS, B, LDB,
     $                INFO )
      ELSE IF( IASCL.EQ.2 ) THEN
         CALL DLASCL( 'G', 0, 0, ANRM, BIGNUM, SCLLEN, NRHS, B, LDB,
     $                INFO )
      END IF
      IF( IBSCL.EQ.1 ) THEN
         CALL DLASCL( 'G', 0, 0, SMLNUM, BNRM, SCLLEN, NRHS, B, LDB,
     $                INFO )
      ELSE IF( IBSCL.EQ.2 ) THEN
         CALL DLASCL( 'G', 0, 0, BIGNUM, BNRM, SCLLEN, NRHS, B, LDB,
     $                INFO )
      END IF
*
   50 CONTINUE
      WORK( 1 ) = DBLE( WSIZE )
*
      RETURN
*
*     End of DGELS
*
      END
