      SUBROUTINE DTRSYL( TRANA, TRANB, ISGN, M, N, A, LDA, B, LDB, C,
     $                   LDC, SCALE, INFO )
*
*  -- LAPACK routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     March 31, 1993
*
*     .. Scalar Arguments ..
      CHARACTER          TRANA, TRANB
      INTEGER            INFO, ISGN, LDA, LDB, LDC, M, N
      DOUBLE PRECISION   SCALE
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), B( LDB, * ), C( LDC, * )
*     ..
*
*  Purpose
*  =======
*
*  DTRSYL solves the real Sylvester matrix equation:
*
*     op(A)*X + X*op(B) = scale*C or
*     op(A)*X - X*op(B) = scale*C,
*
*  where op(A) = A or A**T, and  A and B are both upper quasi-
*  triangular. A is M-by-M and B is N-by-N; the right hand side C and
*  the solution X are M-by-N; and scale is an output scale factor, set
*  <= 1 to avoid overflow in X.
*
*  A and B must be in Schur canonical form (as returned by DHSEQR), that
*  is, block upper triangular with 1-by-1 and 2-by-2 diagonal blocks;
*  each 2-by-2 diagonal block has its diagonal elements equal and its
*  off-diagonal elements of opposite sign.
*
*  Arguments
*  =========
*
*  TRANA   (input) CHARACTER*1
*          Specifies the option op(A):
*          = 'N': op(A) = A    (No transpose)
*          = 'T': op(A) = A**T (Transpose)
*          = 'C': op(A) = A**H (Conjugate transpose = Transpose)
*
*  TRANB   (input) CHARACTER*1
*          Specifies the option op(B):
*          = 'N': op(B) = B    (No transpose)
*          = 'T': op(B) = B**T (Transpose)
*          = 'C': op(B) = B**H (Conjugate transpose = Transpose)
*
*  ISGN    (input) INTEGER
*          Specifies the sign in the equation:
*          = +1: solve op(A)*X + X*op(B) = scale*C
*          = -1: solve op(A)*X - X*op(B) = scale*C
*
*  M       (input) INTEGER
*          The order of the matrix A, and the number of rows in the
*          matrices X and C. M >= 0.
*
*  N       (input) INTEGER
*          The order of the matrix B, and the number of columns in the
*          matrices X and C. N >= 0.
*
*  A       (input) DOUBLE PRECISION array, dimension (LDA,M)
*          The upper quasi-triangular matrix A, in Schur canonical form.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A. LDA >= max(1,M).
*
*  B       (input) DOUBLE PRECISION array, dimension (LDB,N)
*          The upper quasi-triangular matrix B, in Schur canonical form.
*
*  LDB     (input) INTEGER
*          The leading dimension of the array B. LDB >= max(1,N).
*
*  C       (input/output) DOUBLE PRECISION array, dimension (LDC,N)
*          On entry, the M-by-N right hand side matrix C.
*          On exit, C is overwritten by the solution matrix X.
*
*  LDC     (input) INTEGER
*          The leading dimension of the array C. LDC >= max(1,M)
*
*  SCALE   (output) DOUBLE PRECISION
*          The scale factor, scale, set <= 1 to avoid overflow in X.
*
*  INFO    (output) INTEGER
*          = 0: successful exit
*          < 0: if INFO = -i, the i-th argument had an illegal value
*          = 1: A and B have common or very close eigenvalues; perturbed
*               values were used to solve the equation (but the matrices
*               A and B are unchanged).
*
*  =====================================================================
*
*     .. Parameters ..
*     ..
*     .. Local Scalars ..
      LOGICAL            NOTRNA, NOTRNB
      INTEGER            IINFO
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL           XERBLA, SCOPY
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     .. Declare FLAME related stuff
      include "FLAMEf.h"
*
      INTEGER            isgn_obj( FLA_OBJ_SIZE ),
     $                   A_obj( FLA_OBJ_SIZE ),
     $                   B_obj( FLA_OBJ_SIZE ),
     $                   C_obj( FLA_OBJ_SIZE ),
     $                   scale_obj( FLA_OBJ_SIZE ),
     $                   dtype,
     $                   dtype2,
     $                   nscopy
      PARAMETER        ( dtype  = FLA_DOUBLE )
      PARAMETER        ( dtype2 = FLA_DOUBLE )
*     ..
*     .. Executable Statements ..
*
*     Initialize FLAME
*
      CALL FLA_INIT_F( IINFO )
*
*     Decode and Test input parameters
*
      NOTRNA = LSAME( TRANA, 'N' )
      NOTRNB = LSAME( TRANB, 'N' )
*
      INFO = 0
      IF( .NOT.NOTRNA .AND. .NOT.LSAME( TRANA, 'T' ) .AND. .NOT.
     $    LSAME( TRANA, 'C' ) ) THEN
         INFO = -1
      ELSE IF( .NOT.NOTRNB .AND. .NOT.LSAME( TRANB, 'T' ) .AND. .NOT.
     $         LSAME( TRANB, 'C' ) ) THEN
         INFO = -2
      ELSE IF( ISGN.NE.1 .AND. ISGN.NE.-1 ) THEN
         INFO = -3
      ELSE IF( M.LT.0 ) THEN
         INFO = -4
      ELSE IF( N.LT.0 ) THEN
         INFO = -5
      ELSE IF( LDA.LT.MAX( 1, M ) ) THEN
         INFO = -7
      ELSE IF( LDB.LT.MAX( 1, N ) ) THEN
         INFO = -9
      ELSE IF( LDC.LT.MAX( 1, M ) ) THEN
         INFO = -11
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DTRSYL', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( M.EQ.0 .OR. N.EQ.0 )
     $   RETURN
*
*     Assign isgn_obj according to isgn.
*
      nscopy = FLA_OBJ_SIZE / 4
      IF( ISGN.EQ.1 ) THEN
         CALL SCOPY( nscopy, FLA_ONE, 1, isgn_obj, 1 )
      ELSE
         CALL SCOPY( nscopy, FLA_MINUS_ONE, 1, isgn_obj, 1 )
      ENDIF
*
*     Create base object for A_obj and attach matrix buffer.
*
      CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, M, M, A_obj, IINFO )
      CALL FLA_OBJ_ATTACH_BUFF_F( A, LDA, A_obj, IINFO )
*
*     Create base object for A_obj and attach matrix buffer.
*
      CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, N, N, B_obj, IINFO )
      CALL FLA_OBJ_ATTACH_BUFF_F( B, LDB, B_obj, IINFO )
*
*     Create base object for A_obj and attach matrix buffer.
*
      CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, M, N, C_obj, IINFO )
      CALL FLA_OBJ_ATTACH_BUFF_F( C, LDC, C_obj, IINFO )
*
*     Create base object for A_obj and attach matrix buffer.
*
      CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype2, 1, 1, scale_obj, IINFO )
      CALL FLA_OBJ_ATTACH_BUFF_F( scale, 1, scale_obj, IINFO )
*
      IF( NOTRNA .AND. NOTRNB ) THEN
*
*        Solve    A*X + ISGN*X*B = scale*C.
*
         CALL FLA_SYLV_F( FLA_NO_TRANSPOSE, FLA_NO_TRANSPOSE,
     $                    isgn_obj, A_obj, B_obj, C_obj, scale_obj )
*
      ELSE IF( .NOT.NOTRNA .AND. NOTRNB ) THEN
*
*        Solve    A'*X + ISGN*X*B = scale*C.
*
         CALL FLA_SYLV_F( FLA_TRANSPOSE, FLA_NO_TRANSPOSE,
     $                    isgn_obj, A_obj, B_obj, C_obj, scale_obj )
*
      ELSE IF( .NOT.NOTRNA .AND. .NOT.NOTRNB ) THEN
*
*        Solve    A'*X + ISGN*X*B' = scale*C.
*
         CALL FLA_SYLV_F( FLA_TRANSPOSE, FLA_TRANSPOSE,
     $                    isgn_obj, A_obj, B_obj, C_obj, scale_obj )
*
      ELSE IF( NOTRNA .AND. .NOT.NOTRNB ) THEN
*
*        Solve    A*X + ISGN*X*B' = scale*C.
*
         CALL FLA_SYLV_F( FLA_NO_TRANSPOSE, FLA_TRANSPOSE,
     $                    isgn_obj, A_obj, B_obj, C_obj, scale_obj )
*
      END IF
*
*     Free A_obj base object (but leave A buffer alone).
*
      CALL FLA_OBJ_FREE_W_OUT_BUFF_F( A_obj, IINFO )
*
*     Free A_obj base object (but leave A buffer alone).
*
      CALL FLA_OBJ_FREE_W_OUT_BUFF_F( B_obj, IINFO )
*
*     Free A_obj base object (but leave A buffer alone).
*
      CALL FLA_OBJ_FREE_W_OUT_BUFF_F( C_obj, IINFO )
*
*     Free A_obj base object (but leave A buffer alone).
*
      CALL FLA_OBJ_FREE_W_OUT_BUFF_F( scale_obj, IINFO )
*
      RETURN
*
*     End of DTRSYL
*
      END
