/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

#define FLA_ALG_REFERENCE 0
#define FLA_ALG_BLOCKED   1
#define FLA_ALG_UNBLOCKED 2


void time_LU_SA(
               int variant, int type, int n_repeats, int n, int nb_alg,
               FLA_Obj C, FLA_Obj C_ref, FLA_Obj ipiv, 
               double *dtime, double *diff, double *gflops );


int main(int argc, char *argv[])
{
  int 
    m_input, n_input,
    m, n, min_m_n,
    p_first, p_last, p_inc,
    p,
    nb_alg,
    variant,
    uplo,
    n_repeats,
    i, j,
    datatype,
    n_variants = 3;
  
  int  blocksize[16];

  char *colors = "brkgmcbrkg";
  char *ticks  = "o+*xso+*xs";
  char m_dim_desc[14];
  char n_dim_desc[14];
  char m_dim_tag[10];
  char n_dim_tag[10];

  double max_gflops=6.0;

  double
    dtime,
    gflops,
    diff;

  FLA_Obj
    A, B, C, C_ref, ipiv;
  

  /* Initialize FLAME */
  FLA_Init();


  fprintf( stdout, "%c number of repeats:", '%' );
  scanf( "%d", &n_repeats );
  fprintf( stdout, "%c %d\n", '%', n_repeats );

  fprintf( stdout, "%c Enter blocking size:", '%' );
  scanf( "%d", &nb_alg );
  fprintf( stdout, "%c %d\n", '%', nb_alg );

  fprintf( stdout, "%c enter problem size first, last, inc:", '%' );
  scanf( "%d%d%d", &p_first, &p_last, &p_inc );
  fprintf( stdout, "%c %d %d %d\n", '%', p_first, p_last, p_inc );

  fprintf( stdout, "%c enter m n (-1 means bind to problem size): ", '%' );
  scanf( "%d %d", &m_input, &n_input );
  fprintf( stdout, "%c %d %d\n", '%', m_input, n_input );


  /* Delete all existing data structures */
  fprintf( stdout, "\nclear all;\n\n" );


  if     ( m_input >  0 ) {
    sprintf( m_dim_desc, "m = %d", m_input );
    sprintf( m_dim_tag,  "m%dc", m_input);
  }
  else if( m_input <  -1 ) {
    sprintf( m_dim_desc, "m = p/%d", -m_input );
    sprintf( m_dim_tag,  "m%dp", -m_input );
  }
  else if( m_input == -1 ) {
    sprintf( m_dim_desc, "m = p" );
    sprintf( m_dim_tag,  "m%dp", 1 );
  }
  if     ( n_input >  0 ) {
    sprintf( n_dim_desc, "n = %d", n_input );
    sprintf( n_dim_tag,  "n%dc", n_input);
  }
  else if( n_input <  -1 ) {
    sprintf( n_dim_desc, "n = p/%d", -n_input );
    sprintf( n_dim_tag,  "n%dp", -n_input );
  }
  else if( n_input == -1 ) {
    sprintf( n_dim_desc, "n = p" );
    sprintf( n_dim_tag,  "n%dp", 1 );
  }



  i = 1;

  for ( p = p_first; p <= p_last; p += p_inc )
  {

    m = m_input+p;
    n = n_input+p;

    if( m < 0 ) m = p / abs(m_input);
    if( n < 0 ) n = p / abs(n_input);

    min_m_n = min( m, n );

    //datatype = FLA_FLOAT;
    datatype = FLA_DOUBLE;
    //datatype = FLA_COMPLEX;
    //datatype = FLA_DOUBLE_COMPLEX;

    switch( datatype ){

    case FLA_FLOAT:
    {
      FLA_Obj_create( FLA_FLOAT, m, n, &C );
      FLA_Obj_create( FLA_FLOAT, m, n, &C_ref );

      FLA_Random_matrix( C );
      break;
    }

    case FLA_DOUBLE:
    {
      FLA_Obj_create( FLA_DOUBLE, m, n, &C );
      FLA_Obj_create( FLA_DOUBLE, m, n, &C_ref );

      FLA_Random_matrix( C );
      break;
    }

    case FLA_COMPLEX:
    {
      FLA_Obj_create( FLA_COMPLEX, m, n, &C );
      FLA_Obj_create( FLA_COMPLEX, m, n, &C_ref );

      FLA_Random_matrix( C );
      break;
    }

    case FLA_DOUBLE_COMPLEX:
    {
      FLA_Obj_create( FLA_DOUBLE_COMPLEX, m, n, &C );
      FLA_Obj_create( FLA_DOUBLE_COMPLEX, m, n, &C_ref );

      FLA_Random_matrix( C );
      break;
    }

    }

    FLA_Obj_create( FLA_INT, min_m_n, 1, &ipiv );

    FLA_Copy( C, C_ref );

    /* Time the reference implementation */
    time_LU_piv( 0, FLA_ALG_REFERENCE, n_repeats, p, nb_alg,
                 C, C_ref, ipiv, &dtime, &diff, &gflops );

    fprintf( stdout, "data_REF( %d, 1:2 ) = [ %d  %6.3lf ]; \n", i, p, gflops );
    fflush( stdout );

    for ( variant = 3; variant <= 3; variant++ ){
      
      fprintf( stdout, "data_var%d( %d, 1:5 ) = [ %d  ", variant, i, p );
      fflush( stdout );

      time_LU_piv( variant, FLA_ALG_UNBLOCKED, n_repeats, p, nb_alg,
                   C, C_ref, ipiv, &dtime, &diff, &gflops );

      fprintf( stdout, "%6.3lf %6.2le ", gflops, diff );
      fflush( stdout );

      time_LU_piv( variant, FLA_ALG_BLOCKED, n_repeats, p, nb_alg,
                   C, C_ref, ipiv, &dtime, &diff, &gflops );

      fprintf( stdout, "%6.3lf %6.2le ", gflops, diff );
      fflush( stdout );

      fprintf( stdout, " ]; \n" );
      fflush( stdout );
    }

    FLA_Obj_free( &C );
    FLA_Obj_free( &C_ref );
    FLA_Obj_free( &ipiv );
    fprintf( stdout, "\n" );

    i++;
  }

  /* Print the MATLAB commands to plot the data */

  /* Delete all existing figures */
  fprintf( stdout, "figure;\n" );

  /* Plot the data */
  fprintf( stdout, "plot( data_var3( :,1 ), data_var3( :, 2 )./ data_REF( :, 2), '%c:%c' ); \n",
            colors[ 0 ], ticks[ 0 ] );

  printf("hold on\n");

  fprintf( stdout, "plot( data_var3( :,1 ), data_var3( :, 4 )./ data_REF( :, 2), '%c:%c' ); \n",
            colors[ 1 ], ticks[ 1 ] );

  /* Indicate that you want to add to the existing plot */
  fprintf( stdout, "hold on;\n" );

  fprintf( stdout, "legend( 'No reuse of B', ... \n" );
  fprintf( stdout, "        'Reuse B' );\n" );

  fprintf( stdout, "xlabel( 'number of added rows' );\n" );
  fprintf( stdout, "ylabel( 'Speedup' );\n" );
  fprintf( stdout, "title( 'B - %d x %d  ' );\n", m_input, m_input );

  fprintf( stdout, "print -depsc speedup_%d_%d_%d.eps\n", nb_alg, m_input, m_input );
  fprintf( stdout, "hold off;\n");

  FLA_Finalize( );
}

