#
#  libflame
#  An object-based infrastructure for developing high-performance
#  dense linear algebra libraries.
#
#  Copyright (C) 2008, The University of Texas
#
#  libflame is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of
#  the License, or (at your option) any later version.
#
#  libflame is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with libflame; if you did not receive a copy, see
#  http://www.gnu.org/licenses/.
#
#  For more information, please contact us at flame@cs.utexas.edu or
#  send mail to:
#
#  Field G. Van Zee and/or
#  Robert A. van de Geijn
#  The University of Texas at Austin
#  Department of Computer Sciences
#  1 University Station C0500
#  Austin TX 78712
#

#
# --- Include variables determined at configure-time --------------------------
#
CONFIGURE_DEFS = config\config.mk

!if exist ( $(CONFIGURE_DEFS) )
!include $(CONFIGURE_DEFS)
!else
!error nmake: $(CONFIGURE_DEFS) does not exist! Run configure.cmd first.
!endif


#
# --- General build system options --------------------------------------------
#

# Uncomment this for verbose output from nmake.
# VERBOSE = 1

# Uncomment this to enable requisite LAPACK routines to be compiled and included
# into libflame.
ENABLE_BUILTIN_LAPACK = 1

# Assign these variables the full path to your BLAS and LAPACK libraries.
# These variables are only referenced when building libflame as a dynamic
# library. Furthermore, even when buliding a DLL, LAPACK_LIBPATH is not
# referenced if ENABLE_BUILTIN_LAPACK is defined.
BLAS_LIBPATH   = c:\field\lib\blas\blas_WIN32.lib 
LAPACK_LIBPATH = c:\field\lib\lapack\lapack_WIN32.lib 


#
# --- Important build system filenames ----------------------------------------
#

# System and compiler library paths file. The contents of this file should be
# customized when building a dynamically-linked library. The lines of the file
# should be of the form:
#
#   /link /LIBPATH:<path>
#   /link /LIBPATH:<path>
#   /link /LIBPATH:<path>
#   ...
#
# OR
#
#   /link /LIBPATH:<path> /LIBPATH:<path> /LIBPATH:<path> ...
#
# where <path> is the library path to be added to the command line at link time.
LIBPATHS_FILENAME = libpaths.txt
LIBPATHS_FILEPATH = $(PWD)\$(LIBPATHS_FILENAME)

# Various log file names that capture standard output when VERBOSE is undefined.
CC_LOG_FILE   = nmake-cc.log
FC_LOG_FILE   = nmake-fc.log
COPY_LOG_FILE = nmake-copy.log


#
# --- General name and directory definitions -----------------------------------
#

# The relative and absolute locations of the top-level Windows build directory.
# This is the directory in which nmake is run (not the directory named "build").
TOP_BUILD_DIR_REL = .
TOP_BUILD_DIR_ABS = $(PWD)

# The revision string.
REV_STR           = r$(REVISION)

# The names of the libraries.
LIBFLAME_NAME_ONLY        = libflame
LIBFLAME                  = $(LIBFLAME_NAME_ONLY)_$(ARCH_STR)_$(REV_STR)
LIBLAPACK2FLAME_NAME_ONLY = liblapack2flame
LIBLAPACK2FLAME           = $(LIBLAPACK2FLAME_NAME_ONLY)_$(ARCH_STR)_$(REV_STR)

# Directories that reside within the top-level Windows directory.
CNF_DIRNAME       = config
INC_DIRNAME       = include
SRC_DIRNAME       = src
OBJ_DIRNAME       = obj
LIB_DIRNAME       = lib
DLL_DIRNAME       = dll

# Leaves of interest for Windows. We don't build plapack2e code, but we need
# its header files since the base flamec code is currently married to plapack2e.
FLA_DIRNAME       = flamec
L2F_DIRNAME       = lapack2flamec
PLA_DIRNAME       = plapack2e
NET_DIRNAME       = netlib

# Relative directory paths to each of the above subdirectories.
INC_DIRPATH       = $(TOP_BUILD_DIR_REL)\$(INC_DIRNAME)
SRC_DIRPATH       = $(TOP_BUILD_DIR_REL)\$(SRC_DIRNAME)
OBJ_DIRPATH       = $(TOP_BUILD_DIR_REL)\$(OBJ_DIRNAME)
LIB_DIRPATH       = $(TOP_BUILD_DIR_REL)\$(LIB_DIRNAME)
DLL_DIRPATH       = $(TOP_BUILD_DIR_REL)\$(DLL_DIRNAME)

# We only have header files for flamec and plapack2e leaves.
INC_FLA_DIRPATH   = $(INC_DIRPATH)\$(FLA_DIRNAME)
INC_PLA_DIRPATH   = $(INC_DIRPATH)\$(PLA_DIRNAME)

# We have source code for flamec, lapack2flamec, and netlib leaves.
SRC_FLA_DIRPATH   = $(SRC_DIRPATH)\$(FLA_DIRNAME)
SRC_L2F_DIRPATH   = $(SRC_DIRPATH)\$(L2F_DIRNAME)
SRC_NET_DIRPATH   = $(SRC_DIRPATH)\$(NET_DIRNAME)

# And we have object file paths corresponding to those source leaves defined
# above.
OBJ_FLA_DIRPATH   = $(OBJ_DIRPATH)\$(FLA_DIRNAME)\$(ARCH_STR)\$(BUILD_STR)
OBJ_L2F_DIRPATH   = $(OBJ_DIRPATH)\$(L2F_DIRNAME)\$(ARCH_STR)\$(BUILD_STR)
OBJ_NET_DIRPATH   = $(OBJ_DIRPATH)\$(NET_DIRNAME)\$(ARCH_STR)\$(BUILD_STR)

# Separate directories into which we'll move object files when we create the
# static libraries.
LIB_LIBFLAME_DIRPATH        = $(LIB_DIRPATH)\$(ARCH_STR)\$(BUILD_STR)
LIB_LIBLAPACK2FLAME_DIRPATH = $(LIB_DIRPATH)\$(ARCH_STR)\$(BUILD_STR)

# Separate directories into which we'll move object files when we create the
# dynamic libraries.
DLL_LIBFLAME_DIRPATH        = $(DLL_DIRPATH)\$(ARCH_STR)\$(BUILD_STR)
DLL_LIBLAPACK2FLAME_DIRPATH = $(DLL_DIRPATH)\$(ARCH_STR)\$(BUILD_STR)


#
# --- General shell definitions ------------------------------------------------
#

CD     = cd
DIR    = dir
COPY   = copy
DEL    = del /F /Q
MKDIR  = mkdir
RMDIR  = rd /S /Q
ECHO   = echo


#
# --- Compiler-related definitions ---------------------------------------------
#

# Parse options to nmake.
!if "$(BUILD_STR)"=="debug"
CDEBUG = /Zi
FDEBUG = /Zi /dbglibs
COPTIM = /Od
FOPTIM = /Od
!elseif "$(BUILD_STR)"=="release"
CDEBUG =
FDEBUG =
COPTIM = /Ox
FOPTIM = /Ox
!endif

# --- C compiler definitions ---
CC            = icl.exe
CMISCFLAGS    = /nologo
CLANGFLAGS    = /Qstd=c99
CPPROCFLAGS   = /I.\build /I$(INC_FLA_DIRPATH) /I$(INC_PLA_DIRPATH)
CWARNFLAGS    = /w
CDBGFLAGS     = $(CDEBUG)
COPTFLAGS     = $(COPTIM)
CRTIMEFLAGS   = /MT
CMTHREADFLAGS = /Qopenmp
CFLAGS        = $(CMISCFLAGS) $(CLANGFLAGS) $(CPPROCFLAGS) $(CWARNFLAGS) \
                $(CDBGFLAGS) $(COPTFLAGS) $(CRTIMEFLAGS) $(CMTHREADFLAGS)

FC            = ifort.exe
FMISCFLAGS    = /nologo
FLANGFLAGS    = /names:uppercase /assume:nounderscore
FPPROCFLAGS   = /I$(INC_FLA_DIRPATH)
FWARNFLAGS    = /w
FDBGFLAGS     = $(FDEBUG)
FOPTFLAGS     = $(COPTIM)
FRTIMEFLAGS   = /libs:dll /threads
FMTHREADFLAGS = 
FFLAGS        = $(FMISCFLAGS) $(FLANGFLAGS) $(FPPROCFLAGS) $(FWARNFLAGS) \
                $(FDBGFLAGS) $(FOPTFLAGS) $(FRTIMEFLAGS) $(FMTHREADFLAGS)


#
# --- Library-related definitions ----------------------------------------------
#

# --- Static library definitions ---

LIBFLAME_LIB        = $(LIBFLAME).lib
LIBLAPACK2FLAME_LIB = $(LIBLAPACK2FLAME).lib

LIB                 = lib
LIB_OPTIONS         = /nologo
LIB_FLA_OUTPUT_ARG  = /out:$(LIBFLAME_LIB)
LIB_FLA_INPUT_ARGS  = *.obj
LIB_L2F_OUTPUT_ARG  = /out:$(LIBLAPACK2FLAME_LIB)
LIB_L2F_INPUT_ARGS  = *.obj

# --- Dynamic library definitions ---

LIBFLAME_DLL        = $(LIBFLAME).dll

GENDLL              = $(TOP_BUILD_DIR_ABS)\gendll.cmd
FLA_OBJ_LIST_FILE   = libflame_objs.txt
L2F_OBJ_LIST_FILE   = liblapack2flame_objs.txt

